# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Crocube(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field monitor: ax25_frame.payload.ax25_info.monitor
    :field packet_type_q: ax25_frame.payload.ax25_info.packet_type_q
    :field message: ax25_frame.payload.ax25_info.body.message
    
    :field obc_reset_cnt: ax25_frame.payload.ax25_info.body.obc_reset_cnt
    :field obc_uptime: ax25_frame.payload.ax25_info.body.obc_uptime
    :field obc_uptime_tot: ax25_frame.payload.ax25_info.body.obc_uptime_tot
    :field obc_bat: ax25_frame.payload.ax25_info.body.obc_bat
    :field obc_temp_mcu: ax25_frame.payload.ax25_info.body.obc_temp_mcu
    :field obc_freemem: ax25_frame.payload.ax25_info.body.obc_freemem
    
    :field psu_reset_cnt: ax25_frame.payload.ax25_info.body.psu_reset_cnt
    :field psu_uptime: ax25_frame.payload.ax25_info.body.psu_uptime
    :field psu_uptime_tot: ax25_frame.payload.ax25_info.body.psu_uptime_tot
    :field psu_battery: ax25_frame.payload.ax25_info.body.psu_battery
    :field psu_temp_sys: ax25_frame.payload.ax25_info.body.psu_temp_sys
    :field psu_temp_bat: ax25_frame.payload.ax25_info.body.psu_temp_bat
    :field psu_cur_in: ax25_frame.payload.ax25_info.body.psu_cur_in
    :field psu_cur_out: ax25_frame.payload.ax25_info.body.psu_cur_out
    :field psu_ch_state_num: ax25_frame.payload.ax25_info.body.psu_ch_state_num
    :field psu_ch0_state: ax25_frame.payload.ax25_info.body.psu_ch0_state
    :field psu_ch1_state: ax25_frame.payload.ax25_info.body.psu_ch1_state
    :field psu_ch2_state: ax25_frame.payload.ax25_info.body.psu_ch2_state
    :field psu_ch3_state: ax25_frame.payload.ax25_info.body.psu_ch3_state
    :field psu_ch4_state: ax25_frame.payload.ax25_info.body.psu_ch4_state
    :field psu_ch5_state: ax25_frame.payload.ax25_info.body.psu_ch5_state
    :field psu_ch6_state: ax25_frame.payload.ax25_info.body.psu_ch6_state
    :field psu_sys_state: ax25_frame.payload.ax25_info.body.psu_sys_state
    :field psu_gnd_wdt: ax25_frame.payload.ax25_info.body.psu_gnd_wdt
    
    :field mgs_temp_int_mag: ax25_frame.payload.ax25_info.body.mgs_temp_int_mag
    :field mgs_temp_int_gyr: ax25_frame.payload.ax25_info.body.mgs_temp_int_gyr
    :field mgs_int_mag_x: ax25_frame.payload.ax25_info.body.mgs_int_mag_x
    :field mgs_int_mag_y: ax25_frame.payload.ax25_info.body.mgs_int_mag_y
    :field mgs_int_mag_z: ax25_frame.payload.ax25_info.body.mgs_int_mag_z
    :field mgs_int_gyr_x: ax25_frame.payload.ax25_info.body.mgs_int_gyr_x
    :field mgs_int_gyr_y: ax25_frame.payload.ax25_info.body.mgs_int_gyr_y
    :field mgs_int_gyr_z: ax25_frame.payload.ax25_info.body.mgs_int_gyr_z
    :field mgs_temp_ext_mag: ax25_frame.payload.ax25_info.body.mgs_temp_ext_mag
    :field mgs_temp_ext_gyr: ax25_frame.payload.ax25_info.body.mgs_temp_ext_gyr
    :field mgs_ext_mag_x: ax25_frame.payload.ax25_info.body.mgs_ext_mag_x
    :field mgs_ext_mag_y: ax25_frame.payload.ax25_info.body.mgs_ext_mag_y
    :field mgs_ext_mag_z: ax25_frame.payload.ax25_info.body.mgs_ext_mag_z
    :field mgs_ext_gyr_x: ax25_frame.payload.ax25_info.body.mgs_ext_gyr_x
    :field mgs_ext_gyr_y: ax25_frame.payload.ax25_info.body.mgs_ext_gyr_y
    :field mgs_ext_gyr_z: ax25_frame.payload.ax25_info.body.mgs_ext_gyr_z
    
    :field sol_temp_zp: ax25_frame.payload.ax25_info.body.sol_temp_zp
    :field sol_temp_xp: ax25_frame.payload.ax25_info.body.sol_temp_xp
    :field sol_temp_yp: ax25_frame.payload.ax25_info.body.sol_temp_yp
    :field sol_temp_zn: ax25_frame.payload.ax25_info.body.sol_temp_zn
    :field sol_temp_xn: ax25_frame.payload.ax25_info.body.sol_temp_xn
    :field sol_temp_yn: ax25_frame.payload.ax25_info.body.sol_temp_yn
    :field sol_diode_zp: ax25_frame.payload.ax25_info.body.sol_diode_zp
    :field sol_diode_xp: ax25_frame.payload.ax25_info.body.sol_diode_xp
    :field sol_diode_yp: ax25_frame.payload.ax25_info.body.sol_diode_yp
    :field sol_diode_zn: ax25_frame.payload.ax25_info.body.sol_diode_zn
    :field sol_diode_xn: ax25_frame.payload.ax25_info.body.sol_diode_xn
    :field sol_diode_yn: ax25_frame.payload.ax25_info.body.sol_diode_yn
    
    :field atr_master_id: ax25_frame.payload.ax25_info.body.atr_master_id
    :field atr_defective_devices: ax25_frame.payload.ax25_info.body.atr_defective_devices
    :field atr_device_reset_1: ax25_frame.payload.ax25_info.body.atr_device_reset_1
    :field atr_device_reset_2: ax25_frame.payload.ax25_info.body.atr_device_reset_2
    :field atr_device_reset_3: ax25_frame.payload.ax25_info.body.atr_device_reset_3
    :field atr_device_uptime_1: ax25_frame.payload.ax25_info.body.atr_device_uptime_1
    :field atr_device_uptime_2: ax25_frame.payload.ax25_info.body.atr_device_uptime_2
    :field atr_device_uptime_3: ax25_frame.payload.ax25_info.body.atr_device_uptime_3
    :field atr_checksum: ax25_frame.payload.ax25_info.body.atr_checksum
    
    :field uhf_uptime: ax25_frame.payload.ax25_info.body.uhf_uptime
    :field uhf_uptime_tot: ax25_frame.payload.ax25_info.body.uhf_uptime_tot
    :field uhf_reset_cnt: ax25_frame.payload.ax25_info.body.uhf_reset_cnt
    :field uhf_rf_reset_cnt: ax25_frame.payload.ax25_info.body.uhf_rf_reset_cnt
    :field uhf_trx_temp: ax25_frame.payload.ax25_info.body.uhf_trx_temp
    :field uhf_rf_temp: ax25_frame.payload.ax25_info.body.uhf_rf_temp
    :field uhf_pa_temp: ax25_frame.payload.ax25_info.body.uhf_pa_temp
    :field uhf_digipeater_cnt: ax25_frame.payload.ax25_info.body.uhf_digipeater_cnt
    :field uhf_last_digipeater: ax25_frame.payload.ax25_info.body.uhf_last_digipeater
    :field uhf_rx_cnt: ax25_frame.payload.ax25_info.body.uhf_rx_cnt
    :field uhf_tx_cnt: ax25_frame.payload.ax25_info.body.uhf_tx_cnt
    :field uhf_act_rssi_raw: ax25_frame.payload.ax25_info.body.uhf_act_rssi_raw
    :field uhf_dcd_rssi_raw: ax25_frame.payload.ax25_info.body.uhf_dcd_rssi_raw
    
    :field vhf_uptime: ax25_frame.payload.ax25_info.body.vhf_uptime
    :field vhf_uptime_tot: ax25_frame.payload.ax25_info.body.vhf_uptime_tot
    :field vhf_reset_cnt: ax25_frame.payload.ax25_info.body.vhf_reset_cnt
    :field vhf_rf_reset_cnt: ax25_frame.payload.ax25_info.body.vhf_rf_reset_cnt
    :field vhf_trx_temp: ax25_frame.payload.ax25_info.body.vhf_trx_temp
    :field vhf_rf_temp: ax25_frame.payload.ax25_info.body.vhf_rf_temp
    :field vhf_pa_temp: ax25_frame.payload.ax25_info.body.vhf_pa_temp
    :field vhf_digipeater_cnt: ax25_frame.payload.ax25_info.body.vhf_digipeater_cnt
    :field vhf_last_digipeater: ax25_frame.payload.ax25_info.body.vhf_last_digipeater
    :field vhf_rx_cnt: ax25_frame.payload.ax25_info.body.vhf_rx_cnt
    :field vhf_tx_cnt: ax25_frame.payload.ax25_info.body.vhf_tx_cnt
    :field vhf_act_rssi_raw: ax25_frame.payload.ax25_info.body.vhf_act_rssi_raw
    :field vhf_dcd_rssi_raw: ax25_frame.payload.ax25_info.body.vhf_dcd_rssi_raw
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Crocube.Ax25Frame(self._io, self, self._root)

    class Psu(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.psu_pass_packet_type = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_rst_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_uptime_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_uptime_tot_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_bat_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_temp_sys_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_temp_bat_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_cur_in_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_cur_out_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_ch_state_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_sys_state_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.psu_gnd_wdt_str = (self._io.read_bytes_full()).decode(u"utf8")

        @property
        def psu_ch3_state(self):
            if hasattr(self, '_m_psu_ch3_state'):
                return self._m_psu_ch3_state if hasattr(self, '_m_psu_ch3_state') else None

            self._m_psu_ch3_state = ((self.psu_ch_state_num >> 3) & 1)
            return self._m_psu_ch3_state if hasattr(self, '_m_psu_ch3_state') else None

        @property
        def psu_reset_cnt(self):
            if hasattr(self, '_m_psu_reset_cnt'):
                return self._m_psu_reset_cnt if hasattr(self, '_m_psu_reset_cnt') else None

            self._m_psu_reset_cnt = int(self.psu_rst_cnt_str)
            return self._m_psu_reset_cnt if hasattr(self, '_m_psu_reset_cnt') else None

        @property
        def psu_uptime_tot(self):
            if hasattr(self, '_m_psu_uptime_tot'):
                return self._m_psu_uptime_tot if hasattr(self, '_m_psu_uptime_tot') else None

            self._m_psu_uptime_tot = int(self.psu_uptime_tot_str)
            return self._m_psu_uptime_tot if hasattr(self, '_m_psu_uptime_tot') else None

        @property
        def psu_temp_bat(self):
            if hasattr(self, '_m_psu_temp_bat'):
                return self._m_psu_temp_bat if hasattr(self, '_m_psu_temp_bat') else None

            self._m_psu_temp_bat = int(self.psu_temp_bat_str)
            return self._m_psu_temp_bat if hasattr(self, '_m_psu_temp_bat') else None

        @property
        def psu_ch5_state(self):
            if hasattr(self, '_m_psu_ch5_state'):
                return self._m_psu_ch5_state if hasattr(self, '_m_psu_ch5_state') else None

            self._m_psu_ch5_state = ((self.psu_ch_state_num >> 5) & 1)
            return self._m_psu_ch5_state if hasattr(self, '_m_psu_ch5_state') else None

        @property
        def psu_ch0_state(self):
            if hasattr(self, '_m_psu_ch0_state'):
                return self._m_psu_ch0_state if hasattr(self, '_m_psu_ch0_state') else None

            self._m_psu_ch0_state = ((self.psu_ch_state_num >> 0) & 1)
            return self._m_psu_ch0_state if hasattr(self, '_m_psu_ch0_state') else None

        @property
        def psu_gnd_wdt(self):
            if hasattr(self, '_m_psu_gnd_wdt'):
                return self._m_psu_gnd_wdt if hasattr(self, '_m_psu_gnd_wdt') else None

            self._m_psu_gnd_wdt = int(self.psu_gnd_wdt_str)
            return self._m_psu_gnd_wdt if hasattr(self, '_m_psu_gnd_wdt') else None

        @property
        def psu_uptime(self):
            if hasattr(self, '_m_psu_uptime'):
                return self._m_psu_uptime if hasattr(self, '_m_psu_uptime') else None

            self._m_psu_uptime = int(self.psu_uptime_str)
            return self._m_psu_uptime if hasattr(self, '_m_psu_uptime') else None

        @property
        def psu_sys_state(self):
            if hasattr(self, '_m_psu_sys_state'):
                return self._m_psu_sys_state if hasattr(self, '_m_psu_sys_state') else None

            self._m_psu_sys_state = int(self.psu_sys_state_str)
            return self._m_psu_sys_state if hasattr(self, '_m_psu_sys_state') else None

        @property
        def psu_ch_state_num(self):
            if hasattr(self, '_m_psu_ch_state_num'):
                return self._m_psu_ch_state_num if hasattr(self, '_m_psu_ch_state_num') else None

            self._m_psu_ch_state_num = int(self.psu_ch_state_str, 16)
            return self._m_psu_ch_state_num if hasattr(self, '_m_psu_ch_state_num') else None

        @property
        def psu_ch6_state(self):
            if hasattr(self, '_m_psu_ch6_state'):
                return self._m_psu_ch6_state if hasattr(self, '_m_psu_ch6_state') else None

            self._m_psu_ch6_state = ((self.psu_ch_state_num >> 6) & 1)
            return self._m_psu_ch6_state if hasattr(self, '_m_psu_ch6_state') else None

        @property
        def psu_cur_out(self):
            if hasattr(self, '_m_psu_cur_out'):
                return self._m_psu_cur_out if hasattr(self, '_m_psu_cur_out') else None

            self._m_psu_cur_out = int(self.psu_cur_out_str)
            return self._m_psu_cur_out if hasattr(self, '_m_psu_cur_out') else None

        @property
        def psu_ch2_state(self):
            if hasattr(self, '_m_psu_ch2_state'):
                return self._m_psu_ch2_state if hasattr(self, '_m_psu_ch2_state') else None

            self._m_psu_ch2_state = ((self.psu_ch_state_num >> 2) & 1)
            return self._m_psu_ch2_state if hasattr(self, '_m_psu_ch2_state') else None

        @property
        def psu_temp_sys(self):
            if hasattr(self, '_m_psu_temp_sys'):
                return self._m_psu_temp_sys if hasattr(self, '_m_psu_temp_sys') else None

            self._m_psu_temp_sys = int(self.psu_temp_sys_str)
            return self._m_psu_temp_sys if hasattr(self, '_m_psu_temp_sys') else None

        @property
        def psu_cur_in(self):
            if hasattr(self, '_m_psu_cur_in'):
                return self._m_psu_cur_in if hasattr(self, '_m_psu_cur_in') else None

            self._m_psu_cur_in = int(self.psu_cur_in_str)
            return self._m_psu_cur_in if hasattr(self, '_m_psu_cur_in') else None

        @property
        def psu_ch1_state(self):
            if hasattr(self, '_m_psu_ch1_state'):
                return self._m_psu_ch1_state if hasattr(self, '_m_psu_ch1_state') else None

            self._m_psu_ch1_state = ((self.psu_ch_state_num >> 1) & 1)
            return self._m_psu_ch1_state if hasattr(self, '_m_psu_ch1_state') else None

        @property
        def psu_ch4_state(self):
            if hasattr(self, '_m_psu_ch4_state'):
                return self._m_psu_ch4_state if hasattr(self, '_m_psu_ch4_state') else None

            self._m_psu_ch4_state = ((self.psu_ch_state_num >> 4) & 1)
            return self._m_psu_ch4_state if hasattr(self, '_m_psu_ch4_state') else None

        @property
        def psu_battery(self):
            if hasattr(self, '_m_psu_battery'):
                return self._m_psu_battery if hasattr(self, '_m_psu_battery') else None

            self._m_psu_battery = int(self.psu_bat_str)
            return self._m_psu_battery if hasattr(self, '_m_psu_battery') else None


    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Crocube.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Crocube.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Crocube.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Crocube.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Crocube.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Crocube.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Crocube.IFrame(self._io, self, self._root)


    class Mgs(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.mgs_pass_packet_type = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_temp_int_mag_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_temp_int_gyr_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_int_mag_x_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_int_mag_y_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_int_mag_z_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_int_gyr_x_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_int_gyr_y_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_int_gyr_z_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_temp_ext_mag_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_temp_ext_gyr_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_ext_mag_x_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_ext_mag_y_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_ext_mag_z_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_ext_gyr_x_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_ext_gyr_y_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mgs_ext_gyr_z_str = (self._io.read_bytes_full()).decode(u"utf8")

        @property
        def mgs_int_gyr_y(self):
            if hasattr(self, '_m_mgs_int_gyr_y'):
                return self._m_mgs_int_gyr_y if hasattr(self, '_m_mgs_int_gyr_y') else None

            self._m_mgs_int_gyr_y = int(self.mgs_int_gyr_y_str)
            return self._m_mgs_int_gyr_y if hasattr(self, '_m_mgs_int_gyr_y') else None

        @property
        def mgs_ext_mag_x(self):
            if hasattr(self, '_m_mgs_ext_mag_x'):
                return self._m_mgs_ext_mag_x if hasattr(self, '_m_mgs_ext_mag_x') else None

            self._m_mgs_ext_mag_x = int(self.mgs_ext_mag_x_str)
            return self._m_mgs_ext_mag_x if hasattr(self, '_m_mgs_ext_mag_x') else None

        @property
        def mgs_ext_gyr_z(self):
            if hasattr(self, '_m_mgs_ext_gyr_z'):
                return self._m_mgs_ext_gyr_z if hasattr(self, '_m_mgs_ext_gyr_z') else None

            self._m_mgs_ext_gyr_z = int(self.mgs_ext_gyr_z_str)
            return self._m_mgs_ext_gyr_z if hasattr(self, '_m_mgs_ext_gyr_z') else None

        @property
        def mgs_ext_mag_z(self):
            if hasattr(self, '_m_mgs_ext_mag_z'):
                return self._m_mgs_ext_mag_z if hasattr(self, '_m_mgs_ext_mag_z') else None

            self._m_mgs_ext_mag_z = int(self.mgs_ext_mag_z_str)
            return self._m_mgs_ext_mag_z if hasattr(self, '_m_mgs_ext_mag_z') else None

        @property
        def mgs_int_gyr_x(self):
            if hasattr(self, '_m_mgs_int_gyr_x'):
                return self._m_mgs_int_gyr_x if hasattr(self, '_m_mgs_int_gyr_x') else None

            self._m_mgs_int_gyr_x = int(self.mgs_int_gyr_x_str)
            return self._m_mgs_int_gyr_x if hasattr(self, '_m_mgs_int_gyr_x') else None

        @property
        def mgs_temp_ext_gyr(self):
            if hasattr(self, '_m_mgs_temp_ext_gyr'):
                return self._m_mgs_temp_ext_gyr if hasattr(self, '_m_mgs_temp_ext_gyr') else None

            self._m_mgs_temp_ext_gyr = int(self.mgs_temp_ext_gyr_str)
            return self._m_mgs_temp_ext_gyr if hasattr(self, '_m_mgs_temp_ext_gyr') else None

        @property
        def mgs_int_mag_y(self):
            if hasattr(self, '_m_mgs_int_mag_y'):
                return self._m_mgs_int_mag_y if hasattr(self, '_m_mgs_int_mag_y') else None

            self._m_mgs_int_mag_y = int(self.mgs_int_mag_y_str)
            return self._m_mgs_int_mag_y if hasattr(self, '_m_mgs_int_mag_y') else None

        @property
        def mgs_ext_gyr_x(self):
            if hasattr(self, '_m_mgs_ext_gyr_x'):
                return self._m_mgs_ext_gyr_x if hasattr(self, '_m_mgs_ext_gyr_x') else None

            self._m_mgs_ext_gyr_x = int(self.mgs_ext_gyr_x_str)
            return self._m_mgs_ext_gyr_x if hasattr(self, '_m_mgs_ext_gyr_x') else None

        @property
        def mgs_temp_ext_mag(self):
            if hasattr(self, '_m_mgs_temp_ext_mag'):
                return self._m_mgs_temp_ext_mag if hasattr(self, '_m_mgs_temp_ext_mag') else None

            self._m_mgs_temp_ext_mag = int(self.mgs_temp_ext_mag_str)
            return self._m_mgs_temp_ext_mag if hasattr(self, '_m_mgs_temp_ext_mag') else None

        @property
        def mgs_ext_gyr_y(self):
            if hasattr(self, '_m_mgs_ext_gyr_y'):
                return self._m_mgs_ext_gyr_y if hasattr(self, '_m_mgs_ext_gyr_y') else None

            self._m_mgs_ext_gyr_y = int(self.mgs_ext_gyr_y_str)
            return self._m_mgs_ext_gyr_y if hasattr(self, '_m_mgs_ext_gyr_y') else None

        @property
        def mgs_temp_int_gyr(self):
            if hasattr(self, '_m_mgs_temp_int_gyr'):
                return self._m_mgs_temp_int_gyr if hasattr(self, '_m_mgs_temp_int_gyr') else None

            self._m_mgs_temp_int_gyr = int(self.mgs_temp_int_gyr_str)
            return self._m_mgs_temp_int_gyr if hasattr(self, '_m_mgs_temp_int_gyr') else None

        @property
        def mgs_temp_int_mag(self):
            if hasattr(self, '_m_mgs_temp_int_mag'):
                return self._m_mgs_temp_int_mag if hasattr(self, '_m_mgs_temp_int_mag') else None

            self._m_mgs_temp_int_mag = int(self.mgs_temp_int_mag_str)
            return self._m_mgs_temp_int_mag if hasattr(self, '_m_mgs_temp_int_mag') else None

        @property
        def mgs_int_mag_z(self):
            if hasattr(self, '_m_mgs_int_mag_z'):
                return self._m_mgs_int_mag_z if hasattr(self, '_m_mgs_int_mag_z') else None

            self._m_mgs_int_mag_z = int(self.mgs_int_mag_z_str)
            return self._m_mgs_int_mag_z if hasattr(self, '_m_mgs_int_mag_z') else None

        @property
        def mgs_int_mag_x(self):
            if hasattr(self, '_m_mgs_int_mag_x'):
                return self._m_mgs_int_mag_x if hasattr(self, '_m_mgs_int_mag_x') else None

            self._m_mgs_int_mag_x = int(self.mgs_int_mag_x_str)
            return self._m_mgs_int_mag_x if hasattr(self, '_m_mgs_int_mag_x') else None

        @property
        def mgs_ext_mag_y(self):
            if hasattr(self, '_m_mgs_ext_mag_y'):
                return self._m_mgs_ext_mag_y if hasattr(self, '_m_mgs_ext_mag_y') else None

            self._m_mgs_ext_mag_y = int(self.mgs_ext_mag_y_str)
            return self._m_mgs_ext_mag_y if hasattr(self, '_m_mgs_ext_mag_y') else None

        @property
        def mgs_int_gyr_z(self):
            if hasattr(self, '_m_mgs_int_gyr_z'):
                return self._m_mgs_int_gyr_z if hasattr(self, '_m_mgs_int_gyr_z') else None

            self._m_mgs_int_gyr_z = int(self.mgs_int_gyr_z_str)
            return self._m_mgs_int_gyr_z if hasattr(self, '_m_mgs_int_gyr_z') else None


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Crocube.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Crocube.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Crocube.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Crocube.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Crocube.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Crocube.Tlm(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"9A0CC ") or (self.callsign == u"CQ    ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class Uhf(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.uhf_packet_id_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_uptime_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_uptime_tot_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_reset_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_rf_reset_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_trx_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_rf_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_pa_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_digipeater_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_last_digipeater_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_rx_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_tx_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_act_rssi_raw_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uhf_dcd_rssi_raw_str = (self._io.read_bytes_term(0, False, True, True)).decode(u"utf8")

        @property
        def uhf_rf_reset_cnt(self):
            if hasattr(self, '_m_uhf_rf_reset_cnt'):
                return self._m_uhf_rf_reset_cnt if hasattr(self, '_m_uhf_rf_reset_cnt') else None

            self._m_uhf_rf_reset_cnt = int(self.uhf_rf_reset_cnt_str)
            return self._m_uhf_rf_reset_cnt if hasattr(self, '_m_uhf_rf_reset_cnt') else None

        @property
        def uhf_act_rssi_raw(self):
            if hasattr(self, '_m_uhf_act_rssi_raw'):
                return self._m_uhf_act_rssi_raw if hasattr(self, '_m_uhf_act_rssi_raw') else None

            self._m_uhf_act_rssi_raw = int(self.uhf_act_rssi_raw_str)
            return self._m_uhf_act_rssi_raw if hasattr(self, '_m_uhf_act_rssi_raw') else None

        @property
        def uhf_last_digipeater(self):
            if hasattr(self, '_m_uhf_last_digipeater'):
                return self._m_uhf_last_digipeater if hasattr(self, '_m_uhf_last_digipeater') else None

            self._m_uhf_last_digipeater = self.uhf_last_digipeater_str
            return self._m_uhf_last_digipeater if hasattr(self, '_m_uhf_last_digipeater') else None

        @property
        def uhf_pa_temp(self):
            if hasattr(self, '_m_uhf_pa_temp'):
                return self._m_uhf_pa_temp if hasattr(self, '_m_uhf_pa_temp') else None

            self._m_uhf_pa_temp = int(self.uhf_pa_temp_str)
            return self._m_uhf_pa_temp if hasattr(self, '_m_uhf_pa_temp') else None

        @property
        def uhf_uptime_tot(self):
            if hasattr(self, '_m_uhf_uptime_tot'):
                return self._m_uhf_uptime_tot if hasattr(self, '_m_uhf_uptime_tot') else None

            self._m_uhf_uptime_tot = int(self.uhf_uptime_tot_str)
            return self._m_uhf_uptime_tot if hasattr(self, '_m_uhf_uptime_tot') else None

        @property
        def uhf_tx_cnt(self):
            if hasattr(self, '_m_uhf_tx_cnt'):
                return self._m_uhf_tx_cnt if hasattr(self, '_m_uhf_tx_cnt') else None

            self._m_uhf_tx_cnt = int(self.uhf_tx_cnt_str)
            return self._m_uhf_tx_cnt if hasattr(self, '_m_uhf_tx_cnt') else None

        @property
        def uhf_rf_temp(self):
            if hasattr(self, '_m_uhf_rf_temp'):
                return self._m_uhf_rf_temp if hasattr(self, '_m_uhf_rf_temp') else None

            self._m_uhf_rf_temp = int(self.uhf_rf_temp_str)
            return self._m_uhf_rf_temp if hasattr(self, '_m_uhf_rf_temp') else None

        @property
        def uhf_dcd_rssi_raw(self):
            if hasattr(self, '_m_uhf_dcd_rssi_raw'):
                return self._m_uhf_dcd_rssi_raw if hasattr(self, '_m_uhf_dcd_rssi_raw') else None

            self._m_uhf_dcd_rssi_raw = int(self.uhf_dcd_rssi_raw_str)
            return self._m_uhf_dcd_rssi_raw if hasattr(self, '_m_uhf_dcd_rssi_raw') else None

        @property
        def uhf_uptime(self):
            if hasattr(self, '_m_uhf_uptime'):
                return self._m_uhf_uptime if hasattr(self, '_m_uhf_uptime') else None

            self._m_uhf_uptime = int(self.uhf_uptime_str)
            return self._m_uhf_uptime if hasattr(self, '_m_uhf_uptime') else None

        @property
        def uhf_rx_cnt(self):
            if hasattr(self, '_m_uhf_rx_cnt'):
                return self._m_uhf_rx_cnt if hasattr(self, '_m_uhf_rx_cnt') else None

            self._m_uhf_rx_cnt = int(self.uhf_rx_cnt_str)
            return self._m_uhf_rx_cnt if hasattr(self, '_m_uhf_rx_cnt') else None

        @property
        def uhf_reset_cnt(self):
            if hasattr(self, '_m_uhf_reset_cnt'):
                return self._m_uhf_reset_cnt if hasattr(self, '_m_uhf_reset_cnt') else None

            self._m_uhf_reset_cnt = int(self.uhf_reset_cnt_str)
            return self._m_uhf_reset_cnt if hasattr(self, '_m_uhf_reset_cnt') else None

        @property
        def uhf_digipeater_cnt(self):
            if hasattr(self, '_m_uhf_digipeater_cnt'):
                return self._m_uhf_digipeater_cnt if hasattr(self, '_m_uhf_digipeater_cnt') else None

            self._m_uhf_digipeater_cnt = int(self.uhf_digipeater_cnt_str)
            return self._m_uhf_digipeater_cnt if hasattr(self, '_m_uhf_digipeater_cnt') else None

        @property
        def uhf_trx_temp(self):
            if hasattr(self, '_m_uhf_trx_temp'):
                return self._m_uhf_trx_temp if hasattr(self, '_m_uhf_trx_temp') else None

            self._m_uhf_trx_temp = int(self.uhf_trx_temp_str)
            return self._m_uhf_trx_temp if hasattr(self, '_m_uhf_trx_temp') else None


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Crocube.Tlm(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class Vhf(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.vhf_packet_id_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_uptime_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_uptime_tot_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_reset_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_rf_reset_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_trx_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_rf_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_pa_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_digipeater_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_last_digipeater_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_rx_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_tx_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_act_rssi_raw_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.vhf_dcd_rssi_raw_str = (self._io.read_bytes_term(0, False, True, True)).decode(u"utf8")

        @property
        def vhf_digipeater_cnt(self):
            if hasattr(self, '_m_vhf_digipeater_cnt'):
                return self._m_vhf_digipeater_cnt if hasattr(self, '_m_vhf_digipeater_cnt') else None

            self._m_vhf_digipeater_cnt = int(self.vhf_digipeater_cnt_str)
            return self._m_vhf_digipeater_cnt if hasattr(self, '_m_vhf_digipeater_cnt') else None

        @property
        def vhf_tx_cnt(self):
            if hasattr(self, '_m_vhf_tx_cnt'):
                return self._m_vhf_tx_cnt if hasattr(self, '_m_vhf_tx_cnt') else None

            self._m_vhf_tx_cnt = int(self.vhf_tx_cnt_str)
            return self._m_vhf_tx_cnt if hasattr(self, '_m_vhf_tx_cnt') else None

        @property
        def vhf_rx_cnt(self):
            if hasattr(self, '_m_vhf_rx_cnt'):
                return self._m_vhf_rx_cnt if hasattr(self, '_m_vhf_rx_cnt') else None

            self._m_vhf_rx_cnt = int(self.vhf_rx_cnt_str)
            return self._m_vhf_rx_cnt if hasattr(self, '_m_vhf_rx_cnt') else None

        @property
        def vhf_last_digipeater(self):
            if hasattr(self, '_m_vhf_last_digipeater'):
                return self._m_vhf_last_digipeater if hasattr(self, '_m_vhf_last_digipeater') else None

            self._m_vhf_last_digipeater = self.vhf_last_digipeater_str
            return self._m_vhf_last_digipeater if hasattr(self, '_m_vhf_last_digipeater') else None

        @property
        def vhf_pa_temp(self):
            if hasattr(self, '_m_vhf_pa_temp'):
                return self._m_vhf_pa_temp if hasattr(self, '_m_vhf_pa_temp') else None

            self._m_vhf_pa_temp = int(self.vhf_pa_temp_str)
            return self._m_vhf_pa_temp if hasattr(self, '_m_vhf_pa_temp') else None

        @property
        def vhf_uptime(self):
            if hasattr(self, '_m_vhf_uptime'):
                return self._m_vhf_uptime if hasattr(self, '_m_vhf_uptime') else None

            self._m_vhf_uptime = int(self.vhf_uptime_str)
            return self._m_vhf_uptime if hasattr(self, '_m_vhf_uptime') else None

        @property
        def vhf_rf_temp(self):
            if hasattr(self, '_m_vhf_rf_temp'):
                return self._m_vhf_rf_temp if hasattr(self, '_m_vhf_rf_temp') else None

            self._m_vhf_rf_temp = int(self.vhf_rf_temp_str)
            return self._m_vhf_rf_temp if hasattr(self, '_m_vhf_rf_temp') else None

        @property
        def vhf_uptime_tot(self):
            if hasattr(self, '_m_vhf_uptime_tot'):
                return self._m_vhf_uptime_tot if hasattr(self, '_m_vhf_uptime_tot') else None

            self._m_vhf_uptime_tot = int(self.vhf_uptime_tot_str)
            return self._m_vhf_uptime_tot if hasattr(self, '_m_vhf_uptime_tot') else None

        @property
        def vhf_trx_temp(self):
            if hasattr(self, '_m_vhf_trx_temp'):
                return self._m_vhf_trx_temp if hasattr(self, '_m_vhf_trx_temp') else None

            self._m_vhf_trx_temp = int(self.vhf_trx_temp_str)
            return self._m_vhf_trx_temp if hasattr(self, '_m_vhf_trx_temp') else None

        @property
        def vhf_rf_reset_cnt(self):
            if hasattr(self, '_m_vhf_rf_reset_cnt'):
                return self._m_vhf_rf_reset_cnt if hasattr(self, '_m_vhf_rf_reset_cnt') else None

            self._m_vhf_rf_reset_cnt = int(self.vhf_rf_reset_cnt_str)
            return self._m_vhf_rf_reset_cnt if hasattr(self, '_m_vhf_rf_reset_cnt') else None

        @property
        def vhf_dcd_rssi_raw(self):
            if hasattr(self, '_m_vhf_dcd_rssi_raw'):
                return self._m_vhf_dcd_rssi_raw if hasattr(self, '_m_vhf_dcd_rssi_raw') else None

            self._m_vhf_dcd_rssi_raw = int(self.vhf_dcd_rssi_raw_str)
            return self._m_vhf_dcd_rssi_raw if hasattr(self, '_m_vhf_dcd_rssi_raw') else None

        @property
        def vhf_reset_cnt(self):
            if hasattr(self, '_m_vhf_reset_cnt'):
                return self._m_vhf_reset_cnt if hasattr(self, '_m_vhf_reset_cnt') else None

            self._m_vhf_reset_cnt = int(self.vhf_reset_cnt_str)
            return self._m_vhf_reset_cnt if hasattr(self, '_m_vhf_reset_cnt') else None

        @property
        def vhf_act_rssi_raw(self):
            if hasattr(self, '_m_vhf_act_rssi_raw'):
                return self._m_vhf_act_rssi_raw if hasattr(self, '_m_vhf_act_rssi_raw') else None

            self._m_vhf_act_rssi_raw = int(self.vhf_act_rssi_raw_str)
            return self._m_vhf_act_rssi_raw if hasattr(self, '_m_vhf_act_rssi_raw') else None


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Crocube.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Crocube.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Crocube.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class Msg(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.message = (self._io.read_bytes_full()).decode(u"utf8")


    class Obc(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.obc_pass_packet_type = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_rst_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_uptime_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_uptime_tot_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_bat_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_temp_mcu_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_freemem_str = (self._io.read_bytes_full()).decode(u"utf8")

        @property
        def obc_uptime(self):
            if hasattr(self, '_m_obc_uptime'):
                return self._m_obc_uptime if hasattr(self, '_m_obc_uptime') else None

            self._m_obc_uptime = int(self.obc_uptime_str)
            return self._m_obc_uptime if hasattr(self, '_m_obc_uptime') else None

        @property
        def obc_freemem(self):
            if hasattr(self, '_m_obc_freemem'):
                return self._m_obc_freemem if hasattr(self, '_m_obc_freemem') else None

            self._m_obc_freemem = int(self.obc_freemem_str)
            return self._m_obc_freemem if hasattr(self, '_m_obc_freemem') else None

        @property
        def obc_bat(self):
            if hasattr(self, '_m_obc_bat'):
                return self._m_obc_bat if hasattr(self, '_m_obc_bat') else None

            self._m_obc_bat = int(self.obc_bat_str)
            return self._m_obc_bat if hasattr(self, '_m_obc_bat') else None

        @property
        def obc_uptime_tot(self):
            if hasattr(self, '_m_obc_uptime_tot'):
                return self._m_obc_uptime_tot if hasattr(self, '_m_obc_uptime_tot') else None

            self._m_obc_uptime_tot = int(self.obc_uptime_tot_str)
            return self._m_obc_uptime_tot if hasattr(self, '_m_obc_uptime_tot') else None

        @property
        def obc_reset_cnt(self):
            if hasattr(self, '_m_obc_reset_cnt'):
                return self._m_obc_reset_cnt if hasattr(self, '_m_obc_reset_cnt') else None

            self._m_obc_reset_cnt = int(self.obc_rst_cnt_str)
            return self._m_obc_reset_cnt if hasattr(self, '_m_obc_reset_cnt') else None

        @property
        def obc_temp_mcu(self):
            if hasattr(self, '_m_obc_temp_mcu'):
                return self._m_obc_temp_mcu if hasattr(self, '_m_obc_temp_mcu') else None

            self._m_obc_temp_mcu = int(self.obc_temp_mcu_str)
            return self._m_obc_temp_mcu if hasattr(self, '_m_obc_temp_mcu') else None


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Crocube.Callsign(_io__raw_callsign_ror, self, self._root)


    class Atr(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.atr_pass_packet_type = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.atr_master_id_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.atr_defective_devices_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.atr_device_reset_1_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.atr_device_reset_2_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.atr_device_reset_3_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.atr_device_uptime_1_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.atr_device_uptime_2_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.atr_device_uptime_3_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.atr_checksum_str = (self._io.read_bytes_full()).decode(u"utf8")

        @property
        def atr_device_reset_2(self):
            if hasattr(self, '_m_atr_device_reset_2'):
                return self._m_atr_device_reset_2 if hasattr(self, '_m_atr_device_reset_2') else None

            self._m_atr_device_reset_2 = int(self.atr_device_reset_2_str)
            return self._m_atr_device_reset_2 if hasattr(self, '_m_atr_device_reset_2') else None

        @property
        def atr_device_reset_1(self):
            if hasattr(self, '_m_atr_device_reset_1'):
                return self._m_atr_device_reset_1 if hasattr(self, '_m_atr_device_reset_1') else None

            self._m_atr_device_reset_1 = int(self.atr_device_reset_1_str)
            return self._m_atr_device_reset_1 if hasattr(self, '_m_atr_device_reset_1') else None

        @property
        def atr_master_id(self):
            if hasattr(self, '_m_atr_master_id'):
                return self._m_atr_master_id if hasattr(self, '_m_atr_master_id') else None

            self._m_atr_master_id = int(self.atr_master_id_str)
            return self._m_atr_master_id if hasattr(self, '_m_atr_master_id') else None

        @property
        def atr_checksum(self):
            if hasattr(self, '_m_atr_checksum'):
                return self._m_atr_checksum if hasattr(self, '_m_atr_checksum') else None

            self._m_atr_checksum = int(self.atr_checksum_str)
            return self._m_atr_checksum if hasattr(self, '_m_atr_checksum') else None

        @property
        def atr_device_uptime_3(self):
            if hasattr(self, '_m_atr_device_uptime_3'):
                return self._m_atr_device_uptime_3 if hasattr(self, '_m_atr_device_uptime_3') else None

            self._m_atr_device_uptime_3 = int(self.atr_device_uptime_3_str)
            return self._m_atr_device_uptime_3 if hasattr(self, '_m_atr_device_uptime_3') else None

        @property
        def atr_device_reset_3(self):
            if hasattr(self, '_m_atr_device_reset_3'):
                return self._m_atr_device_reset_3 if hasattr(self, '_m_atr_device_reset_3') else None

            self._m_atr_device_reset_3 = int(self.atr_device_reset_3_str)
            return self._m_atr_device_reset_3 if hasattr(self, '_m_atr_device_reset_3') else None

        @property
        def atr_defective_devices(self):
            if hasattr(self, '_m_atr_defective_devices'):
                return self._m_atr_defective_devices if hasattr(self, '_m_atr_defective_devices') else None

            self._m_atr_defective_devices = int(self.atr_defective_devices_str)
            return self._m_atr_defective_devices if hasattr(self, '_m_atr_defective_devices') else None

        @property
        def atr_device_uptime_2(self):
            if hasattr(self, '_m_atr_device_uptime_2'):
                return self._m_atr_device_uptime_2 if hasattr(self, '_m_atr_device_uptime_2') else None

            self._m_atr_device_uptime_2 = int(self.atr_device_uptime_2_str)
            return self._m_atr_device_uptime_2 if hasattr(self, '_m_atr_device_uptime_2') else None

        @property
        def atr_device_uptime_1(self):
            if hasattr(self, '_m_atr_device_uptime_1'):
                return self._m_atr_device_uptime_1 if hasattr(self, '_m_atr_device_uptime_1') else None

            self._m_atr_device_uptime_1 = int(self.atr_device_uptime_1_str)
            return self._m_atr_device_uptime_1 if hasattr(self, '_m_atr_device_uptime_1') else None


    class Tlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.packet_type_q
            if _on == 77:
                self.body = Crocube.Mgs(self._io, self, self._root)
            elif _on == 85:
                self.body = Crocube.Uhf(self._io, self, self._root)
            elif _on == 86:
                self.body = Crocube.Vhf(self._io, self, self._root)
            elif _on == 83:
                self.body = Crocube.Sol(self._io, self, self._root)
            elif _on == 65:
                self.body = Crocube.Atr(self._io, self, self._root)
            elif _on == 79:
                self.body = Crocube.Obc(self._io, self, self._root)
            elif _on == 80:
                self.body = Crocube.Psu(self._io, self, self._root)
            else:
                self.body = Crocube.Msg(self._io, self, self._root)

        @property
        def monitor(self):
            if hasattr(self, '_m_monitor'):
                return self._m_monitor if hasattr(self, '_m_monitor') else None

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_monitor = (self._io.read_bytes_full()).decode(u"ASCII")
            self._io.seek(_pos)
            return self._m_monitor if hasattr(self, '_m_monitor') else None

        @property
        def packet_type_q(self):
            if hasattr(self, '_m_packet_type_q'):
                return self._m_packet_type_q if hasattr(self, '_m_packet_type_q') else None

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_packet_type_q = self._io.read_u1()
            self._io.seek(_pos)
            return self._m_packet_type_q if hasattr(self, '_m_packet_type_q') else None


    class Sol(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sol_pass_packet_type = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_temp_zp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_temp_xp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_temp_yp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_temp_zn_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_temp_xn_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_temp_yn_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_diode_zp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_diode_xp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_diode_yp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_diode_zn_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_diode_xn_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sol_diode_yn_str = (self._io.read_bytes_full()).decode(u"utf8")

        @property
        def sol_temp_zp(self):
            if hasattr(self, '_m_sol_temp_zp'):
                return self._m_sol_temp_zp if hasattr(self, '_m_sol_temp_zp') else None

            self._m_sol_temp_zp = int(self.sol_temp_zp_str)
            return self._m_sol_temp_zp if hasattr(self, '_m_sol_temp_zp') else None

        @property
        def sol_diode_zn(self):
            if hasattr(self, '_m_sol_diode_zn'):
                return self._m_sol_diode_zn if hasattr(self, '_m_sol_diode_zn') else None

            self._m_sol_diode_zn = int(self.sol_diode_zn_str)
            return self._m_sol_diode_zn if hasattr(self, '_m_sol_diode_zn') else None

        @property
        def sol_diode_xp(self):
            if hasattr(self, '_m_sol_diode_xp'):
                return self._m_sol_diode_xp if hasattr(self, '_m_sol_diode_xp') else None

            self._m_sol_diode_xp = int(self.sol_diode_xp_str)
            return self._m_sol_diode_xp if hasattr(self, '_m_sol_diode_xp') else None

        @property
        def sol_temp_zn(self):
            if hasattr(self, '_m_sol_temp_zn'):
                return self._m_sol_temp_zn if hasattr(self, '_m_sol_temp_zn') else None

            self._m_sol_temp_zn = int(self.sol_temp_zn_str)
            return self._m_sol_temp_zn if hasattr(self, '_m_sol_temp_zn') else None

        @property
        def sol_diode_yn(self):
            if hasattr(self, '_m_sol_diode_yn'):
                return self._m_sol_diode_yn if hasattr(self, '_m_sol_diode_yn') else None

            self._m_sol_diode_yn = int(self.sol_diode_yn_str)
            return self._m_sol_diode_yn if hasattr(self, '_m_sol_diode_yn') else None

        @property
        def sol_temp_yn(self):
            if hasattr(self, '_m_sol_temp_yn'):
                return self._m_sol_temp_yn if hasattr(self, '_m_sol_temp_yn') else None

            self._m_sol_temp_yn = int(self.sol_temp_yn_str)
            return self._m_sol_temp_yn if hasattr(self, '_m_sol_temp_yn') else None

        @property
        def sol_diode_zp(self):
            if hasattr(self, '_m_sol_diode_zp'):
                return self._m_sol_diode_zp if hasattr(self, '_m_sol_diode_zp') else None

            self._m_sol_diode_zp = int(self.sol_diode_zp_str)
            return self._m_sol_diode_zp if hasattr(self, '_m_sol_diode_zp') else None

        @property
        def sol_temp_xn(self):
            if hasattr(self, '_m_sol_temp_xn'):
                return self._m_sol_temp_xn if hasattr(self, '_m_sol_temp_xn') else None

            self._m_sol_temp_xn = int(self.sol_temp_xn_str)
            return self._m_sol_temp_xn if hasattr(self, '_m_sol_temp_xn') else None

        @property
        def sol_diode_xn(self):
            if hasattr(self, '_m_sol_diode_xn'):
                return self._m_sol_diode_xn if hasattr(self, '_m_sol_diode_xn') else None

            self._m_sol_diode_xn = int(self.sol_diode_xn_str)
            return self._m_sol_diode_xn if hasattr(self, '_m_sol_diode_xn') else None

        @property
        def sol_diode_yp(self):
            if hasattr(self, '_m_sol_diode_yp'):
                return self._m_sol_diode_yp if hasattr(self, '_m_sol_diode_yp') else None

            self._m_sol_diode_yp = int(self.sol_diode_yp_str)
            return self._m_sol_diode_yp if hasattr(self, '_m_sol_diode_yp') else None

        @property
        def sol_temp_xp(self):
            if hasattr(self, '_m_sol_temp_xp'):
                return self._m_sol_temp_xp if hasattr(self, '_m_sol_temp_xp') else None

            self._m_sol_temp_xp = int(self.sol_temp_xp_str)
            return self._m_sol_temp_xp if hasattr(self, '_m_sol_temp_xp') else None

        @property
        def sol_temp_yp(self):
            if hasattr(self, '_m_sol_temp_yp'):
                return self._m_sol_temp_yp if hasattr(self, '_m_sol_temp_yp') else None

            self._m_sol_temp_yp = int(self.sol_temp_yp_str)
            return self._m_sol_temp_yp if hasattr(self, '_m_sol_temp_yp') else None






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Sakura(KaitaiStruct):
    """:field cw_callsign_and_satellite_name: sakura.type_check.cw_callsign_and_satellite_name
    :field cw_batt_v: sakura.type_check.cw_batt_v
    :field cw_batt_i: sakura.type_check.cw_batt_i
    :field cw_batt_t: sakura.type_check.cw_batt_t
    :field cw_bpb_t: sakura.type_check.cw_bpb_t
    :field cw_raw_i: sakura.type_check.cw_raw_i
    :field cw_power_5v0: sakura.type_check.cw_power_5v0
    :field cw_pwr_ant_dep: sakura.type_check.cw_pwr_ant_dep
    :field cw_power_com: sakura.type_check.cw_power_com
    :field cw_sap_minus_x: sakura.type_check.cw_sap_minus_x
    :field cw_sap_plus_y: sakura.type_check.cw_sap_plus_y
    :field cw_sap_minus_y: sakura.type_check.cw_sap_minus_y
    :field cw_sap_plus_z: sakura.type_check.cw_sap_plus_z
    :field cw_sap_minus_z: sakura.type_check.cw_sap_minus_z
    :field cw_reserve_cmd_counter: sakura.type_check.cw_reserve_cmd_counter
    :field cw_gmsk_cmd_counter: sakura.type_check.cw_gmsk_cmd_counter
    :field cw_kill_sw: sakura.type_check.cw_kill_sw
    :field cw_kill_counter: sakura.type_check.cw_kill_counter
    :field cw_mission_pic_on_off: sakura.type_check.cw_mission_pic_on_off
    :field cw_mis_error_flag: sakura.type_check.cw_mis_error_flag
    :field cw_mis_end_flag: sakura.type_check.cw_mis_end_flag
    :field cw_aprs_flag: sakura.type_check.cw_aprs_flag
    :field cw_current_mis: sakura.type_check.cw_current_mis
    :field cw_beacon: sakura.type_check.cw_beacon
    :field beacon_type: sakura.type_check.beacon_type
    :field cam_callsign_header: sakura.type_check.type_check_1.cam_callsign_header
    :field cam_ctrl: sakura.type_check.type_check_1.cam_ctrl
    :field cam_pid: sakura.type_check.type_check_1.cam_pid
    :field cam_sat_id: sakura.type_check.type_check_1.cam_sat_id
    :field cam_packet_id: sakura.type_check.type_check_1.cam_packet_id
    :field cam_reserved: sakura.type_check.type_check_1.cam_reserved
    :field cam_packet_number: sakura.type_check.type_check_1.cam_packet_number
    :field cam_soi: sakura.type_check.type_check_1.cam_soi
    :field cam_jfif_marker: sakura.type_check.type_check_1.cam_jfif_marker
    :field cam_data_length: sakura.type_check.type_check_1.cam_data_length
    :field cam_jfif_id: sakura.type_check.type_check_1.cam_jfif_id
    :field cam_data_b64_encoded: sakura.type_check.type_check_1.cam_data.b64encstring.cam_data_b64_encoded
    :field beacon_type: sakura.type_check.type_check_1.beacon_type
    :field hk_callsign_header: sakura.type_check.type_check_1.type_check_2.hk_callsign_header
    :field seconds: sakura.type_check.type_check_1.type_check_2.seconds
    :field minutes: sakura.type_check.type_check_1.type_check_2.minutes
    :field hours: sakura.type_check.type_check_1.type_check_2.hours
    :field days: sakura.type_check.type_check_1.type_check_2.days
    :field n_a: sakura.type_check.type_check_1.type_check_2.n_a
    :field temp_minus_x: sakura.type_check.type_check_1.type_check_2.temp_minus_x
    :field temp_plus_y: sakura.type_check.type_check_1.type_check_2.temp_plus_y
    :field temp_minus_y: sakura.type_check.type_check_1.type_check_2.temp_minus_y
    :field temp_plus_z: sakura.type_check.type_check_1.type_check_2.temp_minus_z
    :field temp_minus_z: sakura.type_check.type_check_1.type_check_2.temp_plus_z
    :field bpb_t: sakura.type_check.type_check_1.type_check_2.bpb_t
    :field voltage_minus_x: sakura.type_check.type_check_1.type_check_2.voltage_minus_x
    :field voltage_plus_y: sakura.type_check.type_check_1.type_check_2.voltage_plus_y
    :field voltage_minus_y: sakura.type_check.type_check_1.type_check_2.voltage_minus_y
    :field voltage_plus_z: sakura.type_check.type_check_1.type_check_2.voltage_plus_z
    :field voltage_minus_z: sakura.type_check.type_check_1.type_check_2.voltage_minus_z
    :field current_minus_x: sakura.type_check.type_check_1.type_check_2.current_minus_x
    :field current_plus_y: sakura.type_check.type_check_1.type_check_2.current_plus_y
    :field current_minus_y: sakura.type_check.type_check_1.type_check_2.current_minus_y
    :field current_plus_z: sakura.type_check.type_check_1.type_check_2.current_plus_z
    :field current_minus_z: sakura.type_check.type_check_1.type_check_2.current_plus_z
    :field batt_t: sakura.type_check.type_check_1.type_check_2.batt_t
    :field batt_v: sakura.type_check.type_check_1.type_check_2.batt_v
    :field batt_i: sakura.type_check.type_check_1.type_check_2.batt_i
    :field raw_v: sakura.type_check.type_check_1.type_check_2.raw_v
    :field raw_i: sakura.type_check.type_check_1.type_check_2.raw_i
    :field src_v: sakura.type_check.type_check_1.type_check_2.src_v
    :field src_i: sakura.type_check.type_check_1.type_check_2.src_i
    :field kill_sw: sakura.type_check.type_check_1.type_check_2.kill_sw
    :field raw_v_1: sakura.type_check.type_check_1.type_check_2.raw_v_1
    :field q3v3_1_i: sakura.type_check.type_check_1.type_check_2.q3v3_1_i
    :field q3v3_2_i: sakura.type_check.type_check_1.type_check_2.q3v3_2_i
    :field com_i: sakura.type_check.type_check_1.type_check_2.com_i
    :field ant_dep_i: sakura.type_check.type_check_1.type_check_2.ant_dep_i
    :field q5v0_i: sakura.type_check.type_check_1.type_check_2.q5v0_i
    :field reset_raw_v_mon: sakura.type_check.type_check_1.type_check_2.reset_raw_v_mon
    :field power_com: sakura.type_check.type_check_1.type_check_2.power_com
    :field power_5v0: sakura.type_check.type_check_1.type_check_2.power_5v0
    :field dcdc_3v3_1: sakura.type_check.type_check_1.type_check_2.dcdc_3v3_1
    :field pwr_ant_dep: sakura.type_check.type_check_1.type_check_2.pwr_ant_dep
    :field pwr_3v3_2: sakura.type_check.type_check_1.type_check_2.pwr_3v3_2
    :field pwr_3v3_1: sakura.type_check.type_check_1.type_check_2.pwr_3v3_1
    :field dcdc_5v0: sakura.type_check.type_check_1.type_check_2.dcdc_5v0
    :field dcdc_3v3_2: sakura.type_check.type_check_1.type_check_2.dcdc_3v3_2
    :field pwr_compic: sakura.type_check.type_check_1.type_check_2.pwr_compic
    :field pwr_mainpic: sakura.type_check.type_check_1.type_check_2.pwr_mainpic
    :field empty: sakura.type_check.type_check_1.type_check_2.empty
    :field mp_reset_counter: sakura.type_check.type_check_1.type_check_2.mp_reset_counter
    :field rssi: sakura.type_check.type_check_1.type_check_2.rssi
    :field com_t: sakura.type_check.type_check_1.type_check_2.com_t
    :field com_seq_counter: sakura.type_check.type_check_1.type_check_2.com_seq_counter
    :field cmd_uplink_counter: sakura.type_check.type_check_1.type_check_2.cmd_uplink_counter
    :field mis_ack: sakura.type_check.type_check_1.type_check_2.mis_ack
    :field n_a_2: sakura.type_check.type_check_1.type_check_2.n_a_2
    :field current_mis: sakura.type_check.type_check_1.type_check_2.current_mis
    :field mis_counter: sakura.type_check.type_check_1.type_check_2.mis_counter
    :field checksum: sakura.type_check.type_check_1.type_check_2.checksum
    :field beacon_type: sakura.type_check.type_check_1.type_check_2.beacon_type
    :field auto_gmsk_callsign_header: sakura.type_check.type_check_1.type_check_2.auto_gmsk_callsign_header
    :field flag_data_address: sakura.type_check.type_check_1.type_check_2.flag_data_address
    :field commands_reserved_data_address: sakura.type_check.type_check_1.type_check_2.commands_reserved_data_address
    :field log_data_address: sakura.type_check.type_check_1.type_check_2.log_data_address
    :field hk_data_address: sakura.type_check.type_check_1.type_check_2.hk_data_address
    :field cw_data_address: sakura.type_check.type_check_1.type_check_2.cw_data_address
    :field high_sampling_hk_data_address: sakura.type_check.type_check_1.type_check_2.high_sampling_hk_data_address
    :field aprs_message_data_address: sakura.type_check.type_check_1.type_check_2type_check.aprs_message_data_address
    :field aprs_log_data_address: sakura.type_check.type_check_1.type_check_2.aprs_log_data_address
    :field mission_log_data_address: sakura.type_check.type_check_1.type_check_2.mission_log_data_address
    :field sun_camera_thumbnail_address: sakura.type_check.type_check_1.type_check_2.sun_camera_thumbnail_address
    :field earth_camera_thumbnail_address: sakura.type_check.type_check_1.type_check_2.earth_camera_image_address
    :field sun_camera_image_address: sakura.type_check.type_check_1.type_check_2.sun_camera_image_address
    :field earth_camera_image_address: sakura.type_check.type_check_1.type_check_2.earth_camera_image_address
    :field sun_sensor_log_data_address: sakura.type_check.type_check_1.type_check_2.sun_sensor_log_data_address
    :field flash_memory_rewrite_count: sakura.type_check.type_check_1.type_check_2.flash_memory_rewrite_count
    :field antenna_deployment_attempts_count: sakura.type_check.type_check_1.type_check_2.antenna_deployment_attempts_count
    :field ongoing_mission: sakura.type_check.type_check_1.type_check_2.type_check_1.type_check_2type_check.ongoing_mission
    :field reserved_command: sakura.type_check.type_check_1.type_check_2.reserved_command
    :field kill_switch_flag: sakura.type_check.type_check_1.type_check_2.kill_switch_flag
    :field beacon_type: sakura.type_check.type_check_1.type_check_2.beacon_type
    :field dest_callsign: sakura.type_check.ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: sakura.type_check.ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: sakura.type_check.ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: sakura.type_check.ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_instance___callsign: sakura.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_callsign_raw.callsign_ror.callsign
    :field rpt_instance___ssid: sakura.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.ssid
    :field rpt_instance___hbit: sakura.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.hbit
    :field ctl: sakura.type_check.ax25_frame.ax25_header.ctl
    :field pid: sakura.type_check.ax25_frame.payload.pid
    :field monitor: sakura.type_check.ax25_frame.payload.ax25_info.data_monitor
    :field beacon_type: sakura.type_check.beacon_type
    
    .. seealso::
       Source - https://sites.google.com/p.chibakoudai.jp/gardens-03/home-english/documents/transmission-format
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.sakura = Sakura.SakuraT(self._io, self, self._root)

    class AutoGmskOrHk(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check_2
            if _on == 51:
                self.type_check_2 = Sakura.Hk(self._io, self, self._root)
            else:
                self.type_check_2 = Sakura.AutoGmsk(self._io, self, self._root)

        @property
        def check_2(self):
            if hasattr(self, '_m_check_2'):
                return self._m_check_2 if hasattr(self, '_m_check_2') else None

            _pos = self._io.pos()
            self._io.seek(22)
            self._m_check_2 = self._io.read_u1()
            self._io.seek(_pos)
            return self._m_check_2 if hasattr(self, '_m_check_2') else None


    class Cam(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cam_callsign_header = (self._io.read_bytes(14)).decode(u"ASCII")
            if not self.cam_callsign_header == u"JS1YMY0JS1YNI0":
                raise kaitaistruct.ValidationNotEqualError(u"JS1YMY0JS1YNI0", self.cam_callsign_header, self._io, u"/types/cam/seq/0")
            self.cam_ctrl = self._io.read_u1()
            if not self.cam_ctrl == 62:
                raise kaitaistruct.ValidationNotEqualError(62, self.cam_ctrl, self._io, u"/types/cam/seq/1")
            self.cam_pid = self._io.read_u1()
            if not self.cam_pid == 240:
                raise kaitaistruct.ValidationNotEqualError(240, self.cam_pid, self._io, u"/types/cam/seq/2")
            self.cam_sat_id = self._io.read_u1()
            if not self.cam_sat_id == 83:
                raise kaitaistruct.ValidationNotEqualError(83, self.cam_sat_id, self._io, u"/types/cam/seq/3")
            self.cam_packet_id = self._io.read_u1()
            if not self.cam_packet_id == 204:
                raise kaitaistruct.ValidationNotEqualError(204, self.cam_packet_id, self._io, u"/types/cam/seq/4")
            self.cam_reserved = self._io.read_u1()
            self.packet_number_byte1 = self._io.read_u1()
            self.packet_number_byte2 = self._io.read_u1()
            self.packet_number_byte3 = self._io.read_u1()
            self.cam_soi = self._io.read_u2be()
            if not self.cam_soi == 65496:
                raise kaitaistruct.ValidationNotEqualError(65496, self.cam_soi, self._io, u"/types/cam/seq/9")
            self.cam_jfif_marker = self._io.read_u2be()
            if not self.cam_jfif_marker == 65504:
                raise kaitaistruct.ValidationNotEqualError(65504, self.cam_jfif_marker, self._io, u"/types/cam/seq/10")
            self.cam_data_length = self._io.read_u2be()
            self.cam_jfif_id = (self._io.read_bytes(4)).decode(u"ASCII")
            if not self.cam_jfif_id == u"JFIF":
                raise kaitaistruct.ValidationNotEqualError(u"JFIF", self.cam_jfif_id, self._io, u"/types/cam/seq/12")
            self._raw_cam_data = self._io.read_bytes_full()
            _io__raw_cam_data = KaitaiStream(BytesIO(self._raw_cam_data))
            self.cam_data = Sakura.Cam.Base64(_io__raw_cam_data, self, self._root)

        class Base64(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_b64encstring = self._io.read_bytes_full()
                _process = satnogsdecoders.process.B64encode()
                self._raw_b64encstring = _process.decode(self._raw__raw_b64encstring)
                _io__raw_b64encstring = KaitaiStream(BytesIO(self._raw_b64encstring))
                self.b64encstring = Sakura.Cam.Base64string(_io__raw_b64encstring, self, self._root)


        class Base64string(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.cam_data_b64_encoded = (self._io.read_bytes_full()).decode(u"UTF-8")


        @property
        def cam_packet_number(self):
            if hasattr(self, '_m_cam_packet_number'):
                return self._m_cam_packet_number if hasattr(self, '_m_cam_packet_number') else None

            self._m_cam_packet_number = (((self.packet_number_byte1 << 16) | (self.packet_number_byte2 << 8)) | self.packet_number_byte3)
            return self._m_cam_packet_number if hasattr(self, '_m_cam_packet_number') else None

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"CAM" if 0 == 0 else u"CAM")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None


    class AutoGmsk(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.auto_gmsk_callsign_header = (self._io.read_bytes(14)).decode(u"ASCII")
            if not self.auto_gmsk_callsign_header == u"JS1YMY0JS1YNI0":
                raise kaitaistruct.ValidationNotEqualError(u"JS1YMY0JS1YNI0", self.auto_gmsk_callsign_header, self._io, u"/types/auto_gmsk/seq/0")
            self.auto_gmsk_header_last_part = self._io.read_u2be()
            if not self.auto_gmsk_header_last_part == 16112:
                raise kaitaistruct.ValidationNotEqualError(16112, self.auto_gmsk_header_last_part, self._io, u"/types/auto_gmsk/seq/1")
            self.auto_gmsk_header_last_part_1 = self._io.read_u2be()
            if not self.auto_gmsk_header_last_part_1 == 21265:
                raise kaitaistruct.ValidationNotEqualError(21265, self.auto_gmsk_header_last_part_1, self._io, u"/types/auto_gmsk/seq/2")
            self.auto_gmsk_header_last_part_2 = self._io.read_u2be()
            if not self.auto_gmsk_header_last_part_2 == 65280:
                raise kaitaistruct.ValidationNotEqualError(65280, self.auto_gmsk_header_last_part_2, self._io, u"/types/auto_gmsk/seq/3")
            self.auto_gmsk_header_last_part_3 = self._io.read_u2be()
            if not self.auto_gmsk_header_last_part_3 == 1:
                raise kaitaistruct.ValidationNotEqualError(1, self.auto_gmsk_header_last_part_3, self._io, u"/types/auto_gmsk/seq/4")
            self.flag_data_address = self._io.read_u4be()
            self.commands_reserved_data_address = self._io.read_u4be()
            self.log_data_address = self._io.read_u4be()
            self.hk_data_address = self._io.read_u4be()
            self.cw_data_address = self._io.read_u4be()
            self.high_sampling_hk_data_address = self._io.read_u4be()
            self.aprs_message_data_address = self._io.read_u4be()
            self.aprs_log_data_address = self._io.read_u4be()
            self.mission_log_data_address = self._io.read_u4be()
            self.sun_camera_thumbnail_address = self._io.read_u4be()
            self.earth_camera_thumbnail_address = self._io.read_u4be()
            self.sun_camera_image_address = self._io.read_u4be()
            self.earth_camera_image_address = self._io.read_u4be()
            self.sun_sensor_log_data_address = self._io.read_u4be()
            self.flash_memory_rewrite_count = self._io.read_u4be()
            self.antenna_deployment_attempts_count = self._io.read_u1()
            self.ongoing_mission = self._io.read_u1()
            self.reserved_command = self._io.read_u1()
            self.kill_switch_flag = self._io.read_u1()

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"AUTO GMSK" if 0 == 0 else u"AUTO GMSK")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None


    class Discard(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.first_byte = self._io.read_u1()
            if not self.first_byte == 0:
                raise kaitaistruct.ValidationNotEqualError(0, self.first_byte, self._io, u"/types/discard/seq/0")


    class Cw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cw_callsign_and_satellite_name = (self._io.read_bytes(21)).decode(u"ASCII")
            if not self.cw_callsign_and_satellite_name == u"sakura js1ymy js1yni ":
                raise kaitaistruct.ValidationNotEqualError(u"sakura js1ymy js1yni ", self.cw_callsign_and_satellite_name, self._io, u"/types/cw/seq/0")
            self.cw_batt_v = self._io.read_u1()
            self.cw_batt_i = self._io.read_u1()
            self.cw_batt_t = self._io.read_u1()
            self.cw_bpb_t = self._io.read_u1()
            self.cw_raw_i = self._io.read_u1()
            self.cw_power_5v0 = self._io.read_bits_int_be(1) != 0
            self.cw_pwr_ant_dep = self._io.read_bits_int_be(1) != 0
            self.cw_power_com = self._io.read_bits_int_be(1) != 0
            self.cw_sap_minus_x = self._io.read_bits_int_be(1) != 0
            self.cw_sap_plus_y = self._io.read_bits_int_be(1) != 0
            self.cw_sap_minus_y = self._io.read_bits_int_be(1) != 0
            self.cw_sap_plus_z = self._io.read_bits_int_be(1) != 0
            self.cw_sap_minus_z = self._io.read_bits_int_be(1) != 0
            self.cw_reserve_cmd_counter = self._io.read_bits_int_be(4)
            self.cw_gmsk_cmd_counter = self._io.read_bits_int_be(3)
            self.cw_kill_sw = self._io.read_bits_int_be(1) != 0
            self.cw_kill_counter = self._io.read_bits_int_be(2)
            self.cw_mission_pic_on_off = self._io.read_bits_int_be(1) != 0
            self.cw_mis_error_flag = self._io.read_bits_int_be(1) != 0
            self.cw_mis_end_flag = self._io.read_bits_int_be(1) != 0
            self.cw_aprs_flag = self._io.read_bits_int_be(1) != 0
            self.cw_current_mis = self._io.read_bits_int_be(2)

        @property
        def data_1_dec_value_5(self):
            if hasattr(self, '_m_data_1_dec_value_5'):
                return self._m_data_1_dec_value_5 if hasattr(self, '_m_data_1_dec_value_5') else None

            self._m_data_1_dec_value_5 = (8 if int(self.cw_sap_plus_y) == 1 else 0)
            return self._m_data_1_dec_value_5 if hasattr(self, '_m_data_1_dec_value_5') else None

        @property
        def raw_i_hex_left_digit(self):
            if hasattr(self, '_m_raw_i_hex_left_digit'):
                return self._m_raw_i_hex_left_digit if hasattr(self, '_m_raw_i_hex_left_digit') else None

            self._m_raw_i_hex_left_digit = (u"a" if str(self.raw_i_hex_left) == u"10" else (u"b" if str(self.raw_i_hex_left) == u"11" else (u"c" if str(self.raw_i_hex_left) == u"12" else (u"d" if str(self.raw_i_hex_left) == u"13" else (u"e" if str(self.raw_i_hex_left) == u"14" else (u"f" if str(self.raw_i_hex_left) == u"15" else str(self.raw_i_hex_left)))))))
            return self._m_raw_i_hex_left_digit if hasattr(self, '_m_raw_i_hex_left_digit') else None

        @property
        def data_1_dec(self):
            if hasattr(self, '_m_data_1_dec'):
                return self._m_data_1_dec if hasattr(self, '_m_data_1_dec') else None

            self._m_data_1_dec = (((((((self.data_1_dec_value_1 + self.data_1_dec_value_2) + self.data_1_dec_value_3) + self.data_1_dec_value_4) + self.data_1_dec_value_5) + self.data_1_dec_value_6) + self.data_1_dec_value_7) + self.data_1_dec_value_8)
            return self._m_data_1_dec if hasattr(self, '_m_data_1_dec') else None

        @property
        def data_1_dec_value_4(self):
            if hasattr(self, '_m_data_1_dec_value_4'):
                return self._m_data_1_dec_value_4 if hasattr(self, '_m_data_1_dec_value_4') else None

            self._m_data_1_dec_value_4 = (16 if int(self.cw_sap_minus_x) == 1 else 0)
            return self._m_data_1_dec_value_4 if hasattr(self, '_m_data_1_dec_value_4') else None

        @property
        def batt_t_hex(self):
            if hasattr(self, '_m_batt_t_hex'):
                return self._m_batt_t_hex if hasattr(self, '_m_batt_t_hex') else None

            self._m_batt_t_hex = self.batt_t_hex_left_digit + self.batt_t_hex_right_digit
            return self._m_batt_t_hex if hasattr(self, '_m_batt_t_hex') else None

        @property
        def data_2_hex_left(self):
            if hasattr(self, '_m_data_2_hex_left'):
                return self._m_data_2_hex_left if hasattr(self, '_m_data_2_hex_left') else None

            self._m_data_2_hex_left = self.data_2_dec // 16
            return self._m_data_2_hex_left if hasattr(self, '_m_data_2_hex_left') else None

        @property
        def data_3_hex_right_digit(self):
            if hasattr(self, '_m_data_3_hex_right_digit'):
                return self._m_data_3_hex_right_digit if hasattr(self, '_m_data_3_hex_right_digit') else None

            self._m_data_3_hex_right_digit = (u"a" if str(self.data_3_hex_right) == u"10" else (u"b" if str(self.data_3_hex_right) == u"11" else (u"c" if str(self.data_3_hex_right) == u"12" else (u"d" if str(self.data_3_hex_right) == u"13" else (u"e" if str(self.data_3_hex_right) == u"14" else (u"f" if str(self.data_3_hex_right) == u"15" else str(self.data_3_hex_right)))))))
            return self._m_data_3_hex_right_digit if hasattr(self, '_m_data_3_hex_right_digit') else None

        @property
        def data_1_dec_value_6(self):
            if hasattr(self, '_m_data_1_dec_value_6'):
                return self._m_data_1_dec_value_6 if hasattr(self, '_m_data_1_dec_value_6') else None

            self._m_data_1_dec_value_6 = (4 if int(self.cw_sap_minus_y) == 1 else 0)
            return self._m_data_1_dec_value_6 if hasattr(self, '_m_data_1_dec_value_6') else None

        @property
        def raw_i_hex(self):
            if hasattr(self, '_m_raw_i_hex'):
                return self._m_raw_i_hex if hasattr(self, '_m_raw_i_hex') else None

            self._m_raw_i_hex = self.raw_i_hex_left_digit + self.raw_i_hex_right_digit
            return self._m_raw_i_hex if hasattr(self, '_m_raw_i_hex') else None

        @property
        def batt_i_hex(self):
            if hasattr(self, '_m_batt_i_hex'):
                return self._m_batt_i_hex if hasattr(self, '_m_batt_i_hex') else None

            self._m_batt_i_hex = self.batt_i_hex_left_digit + self.batt_i_hex_right_digit
            return self._m_batt_i_hex if hasattr(self, '_m_batt_i_hex') else None

        @property
        def data_1_hex_left(self):
            if hasattr(self, '_m_data_1_hex_left'):
                return self._m_data_1_hex_left if hasattr(self, '_m_data_1_hex_left') else None

            self._m_data_1_hex_left = self.data_1_dec // 16
            return self._m_data_1_hex_left if hasattr(self, '_m_data_1_hex_left') else None

        @property
        def data_1_dec_value_7(self):
            if hasattr(self, '_m_data_1_dec_value_7'):
                return self._m_data_1_dec_value_7 if hasattr(self, '_m_data_1_dec_value_7') else None

            self._m_data_1_dec_value_7 = (2 if int(self.cw_sap_plus_z) == 1 else 0)
            return self._m_data_1_dec_value_7 if hasattr(self, '_m_data_1_dec_value_7') else None

        @property
        def data_3_hex(self):
            if hasattr(self, '_m_data_3_hex'):
                return self._m_data_3_hex if hasattr(self, '_m_data_3_hex') else None

            self._m_data_3_hex = self.data_3_hex_left_digit + self.data_3_hex_right_digit
            return self._m_data_3_hex if hasattr(self, '_m_data_3_hex') else None

        @property
        def data_2_dec(self):
            if hasattr(self, '_m_data_2_dec'):
                return self._m_data_2_dec if hasattr(self, '_m_data_2_dec') else None

            self._m_data_2_dec = ((self.data_2_dec_value_1 + self.data_2_dec_value_2) + self.data_2_dec_value_3)
            return self._m_data_2_dec if hasattr(self, '_m_data_2_dec') else None

        @property
        def data_1_hex_left_digit(self):
            if hasattr(self, '_m_data_1_hex_left_digit'):
                return self._m_data_1_hex_left_digit if hasattr(self, '_m_data_1_hex_left_digit') else None

            self._m_data_1_hex_left_digit = (u"a" if str(self.data_1_hex_left) == u"10" else (u"b" if str(self.data_1_hex_left) == u"11" else (u"c" if str(self.data_1_hex_left) == u"12" else (u"d" if str(self.data_1_hex_left) == u"13" else (u"e" if str(self.data_1_hex_left) == u"14" else (u"f" if str(self.data_1_hex_left) == u"15" else str(self.data_1_hex_left)))))))
            return self._m_data_1_hex_left_digit if hasattr(self, '_m_data_1_hex_left_digit') else None

        @property
        def data_2_dec_value_2(self):
            if hasattr(self, '_m_data_2_dec_value_2'):
                return self._m_data_2_dec_value_2 if hasattr(self, '_m_data_2_dec_value_2') else None

            self._m_data_2_dec_value_2 = (self.cw_gmsk_cmd_counter * 2)
            return self._m_data_2_dec_value_2 if hasattr(self, '_m_data_2_dec_value_2') else None

        @property
        def data_3_hex_right(self):
            if hasattr(self, '_m_data_3_hex_right'):
                return self._m_data_3_hex_right if hasattr(self, '_m_data_3_hex_right') else None

            self._m_data_3_hex_right = (self.data_3_dec % 16)
            return self._m_data_3_hex_right if hasattr(self, '_m_data_3_hex_right') else None

        @property
        def bpb_t_hex_right_digit(self):
            if hasattr(self, '_m_bpb_t_hex_right_digit'):
                return self._m_bpb_t_hex_right_digit if hasattr(self, '_m_bpb_t_hex_right_digit') else None

            self._m_bpb_t_hex_right_digit = (u"a" if str(self.bpb_t_hex_right) == u"10" else (u"b" if str(self.bpb_t_hex_right) == u"11" else (u"c" if str(self.bpb_t_hex_right) == u"12" else (u"d" if str(self.bpb_t_hex_right) == u"13" else (u"e" if str(self.bpb_t_hex_right) == u"14" else (u"f" if str(self.bpb_t_hex_right) == u"15" else str(self.bpb_t_hex_right)))))))
            return self._m_bpb_t_hex_right_digit if hasattr(self, '_m_bpb_t_hex_right_digit') else None

        @property
        def batt_t_hex_left_digit(self):
            if hasattr(self, '_m_batt_t_hex_left_digit'):
                return self._m_batt_t_hex_left_digit if hasattr(self, '_m_batt_t_hex_left_digit') else None

            self._m_batt_t_hex_left_digit = (u"a" if str(self.batt_t_hex_left) == u"10" else (u"b" if str(self.batt_t_hex_left) == u"11" else (u"c" if str(self.batt_t_hex_left) == u"12" else (u"d" if str(self.batt_t_hex_left) == u"13" else (u"e" if str(self.batt_t_hex_left) == u"14" else (u"f" if str(self.batt_t_hex_left) == u"15" else str(self.batt_t_hex_left)))))))
            return self._m_batt_t_hex_left_digit if hasattr(self, '_m_batt_t_hex_left_digit') else None

        @property
        def data_2_dec_value_1(self):
            if hasattr(self, '_m_data_2_dec_value_1'):
                return self._m_data_2_dec_value_1 if hasattr(self, '_m_data_2_dec_value_1') else None

            self._m_data_2_dec_value_1 = (self.cw_reserve_cmd_counter * 16)
            return self._m_data_2_dec_value_1 if hasattr(self, '_m_data_2_dec_value_1') else None

        @property
        def data_3_dec_value_3(self):
            if hasattr(self, '_m_data_3_dec_value_3'):
                return self._m_data_3_dec_value_3 if hasattr(self, '_m_data_3_dec_value_3') else None

            self._m_data_3_dec_value_3 = (16 if int(self.cw_mis_error_flag) == 1 else 0)
            return self._m_data_3_dec_value_3 if hasattr(self, '_m_data_3_dec_value_3') else None

        @property
        def data_1_dec_value_1(self):
            if hasattr(self, '_m_data_1_dec_value_1'):
                return self._m_data_1_dec_value_1 if hasattr(self, '_m_data_1_dec_value_1') else None

            self._m_data_1_dec_value_1 = (128 if int(self.cw_power_5v0) == 1 else 0)
            return self._m_data_1_dec_value_1 if hasattr(self, '_m_data_1_dec_value_1') else None

        @property
        def data_3_dec_value_6(self):
            if hasattr(self, '_m_data_3_dec_value_6'):
                return self._m_data_3_dec_value_6 if hasattr(self, '_m_data_3_dec_value_6') else None

            self._m_data_3_dec_value_6 = self.cw_current_mis
            return self._m_data_3_dec_value_6 if hasattr(self, '_m_data_3_dec_value_6') else None

        @property
        def batt_i_hex_right_digit(self):
            if hasattr(self, '_m_batt_i_hex_right_digit'):
                return self._m_batt_i_hex_right_digit if hasattr(self, '_m_batt_i_hex_right_digit') else None

            self._m_batt_i_hex_right_digit = (u"a" if str(self.batt_i_hex_right) == u"10" else (u"b" if str(self.batt_i_hex_right) == u"11" else (u"c" if str(self.batt_i_hex_right) == u"12" else (u"d" if str(self.batt_i_hex_right) == u"13" else (u"e" if str(self.batt_i_hex_right) == u"14" else (u"f" if str(self.batt_i_hex_right) == u"15" else str(self.batt_i_hex_right)))))))
            return self._m_batt_i_hex_right_digit if hasattr(self, '_m_batt_i_hex_right_digit') else None

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"CW" if 0 == 0 else u"CW")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

        @property
        def data_3_dec_value_4(self):
            if hasattr(self, '_m_data_3_dec_value_4'):
                return self._m_data_3_dec_value_4 if hasattr(self, '_m_data_3_dec_value_4') else None

            self._m_data_3_dec_value_4 = (8 if int(self.cw_mis_end_flag) == 1 else 0)
            return self._m_data_3_dec_value_4 if hasattr(self, '_m_data_3_dec_value_4') else None

        @property
        def batt_v_hex_left_digit(self):
            if hasattr(self, '_m_batt_v_hex_left_digit'):
                return self._m_batt_v_hex_left_digit if hasattr(self, '_m_batt_v_hex_left_digit') else None

            self._m_batt_v_hex_left_digit = (u"a" if str(self.batt_v_hex_left) == u"10" else (u"b" if str(self.batt_v_hex_left) == u"11" else (u"c" if str(self.batt_v_hex_left) == u"12" else (u"d" if str(self.batt_v_hex_left) == u"13" else (u"e" if str(self.batt_v_hex_left) == u"14" else (u"f" if str(self.batt_v_hex_left) == u"15" else str(self.batt_v_hex_left)))))))
            return self._m_batt_v_hex_left_digit if hasattr(self, '_m_batt_v_hex_left_digit') else None

        @property
        def data_2_hex_right(self):
            if hasattr(self, '_m_data_2_hex_right'):
                return self._m_data_2_hex_right if hasattr(self, '_m_data_2_hex_right') else None

            self._m_data_2_hex_right = (self.data_2_dec % 16)
            return self._m_data_2_hex_right if hasattr(self, '_m_data_2_hex_right') else None

        @property
        def data_3_dec_value_1(self):
            if hasattr(self, '_m_data_3_dec_value_1'):
                return self._m_data_3_dec_value_1 if hasattr(self, '_m_data_3_dec_value_1') else None

            self._m_data_3_dec_value_1 = (self.cw_kill_counter * 64)
            return self._m_data_3_dec_value_1 if hasattr(self, '_m_data_3_dec_value_1') else None

        @property
        def batt_t_hex_right(self):
            if hasattr(self, '_m_batt_t_hex_right'):
                return self._m_batt_t_hex_right if hasattr(self, '_m_batt_t_hex_right') else None

            self._m_batt_t_hex_right = (self.cw_batt_t % 16)
            return self._m_batt_t_hex_right if hasattr(self, '_m_batt_t_hex_right') else None

        @property
        def data_1_dec_value_2(self):
            if hasattr(self, '_m_data_1_dec_value_2'):
                return self._m_data_1_dec_value_2 if hasattr(self, '_m_data_1_dec_value_2') else None

            self._m_data_1_dec_value_2 = (64 if int(self.cw_pwr_ant_dep) == 1 else 0)
            return self._m_data_1_dec_value_2 if hasattr(self, '_m_data_1_dec_value_2') else None

        @property
        def data_3_dec_value_2(self):
            if hasattr(self, '_m_data_3_dec_value_2'):
                return self._m_data_3_dec_value_2 if hasattr(self, '_m_data_3_dec_value_2') else None

            self._m_data_3_dec_value_2 = (32 if int(self.cw_mission_pic_on_off) == 1 else 0)
            return self._m_data_3_dec_value_2 if hasattr(self, '_m_data_3_dec_value_2') else None

        @property
        def data_2_hex_left_digit(self):
            if hasattr(self, '_m_data_2_hex_left_digit'):
                return self._m_data_2_hex_left_digit if hasattr(self, '_m_data_2_hex_left_digit') else None

            self._m_data_2_hex_left_digit = (u"a" if str(self.data_2_hex_left) == u"10" else (u"b" if str(self.data_2_hex_left) == u"11" else (u"c" if str(self.data_2_hex_left) == u"12" else (u"d" if str(self.data_2_hex_left) == u"13" else (u"e" if str(self.data_2_hex_left) == u"14" else (u"f" if str(self.data_2_hex_left) == u"15" else str(self.data_2_hex_left)))))))
            return self._m_data_2_hex_left_digit if hasattr(self, '_m_data_2_hex_left_digit') else None

        @property
        def batt_i_hex_left_digit(self):
            if hasattr(self, '_m_batt_i_hex_left_digit'):
                return self._m_batt_i_hex_left_digit if hasattr(self, '_m_batt_i_hex_left_digit') else None

            self._m_batt_i_hex_left_digit = (u"a" if str(self.batt_i_hex_left) == u"10" else (u"b" if str(self.batt_i_hex_left) == u"11" else (u"c" if str(self.batt_i_hex_left) == u"12" else (u"d" if str(self.batt_i_hex_left) == u"13" else (u"e" if str(self.batt_i_hex_left) == u"14" else (u"f" if str(self.batt_i_hex_left) == u"15" else str(self.batt_i_hex_left)))))))
            return self._m_batt_i_hex_left_digit if hasattr(self, '_m_batt_i_hex_left_digit') else None

        @property
        def data_2_hex_right_digit(self):
            if hasattr(self, '_m_data_2_hex_right_digit'):
                return self._m_data_2_hex_right_digit if hasattr(self, '_m_data_2_hex_right_digit') else None

            self._m_data_2_hex_right_digit = (u"a" if str(self.data_2_hex_right) == u"10" else (u"b" if str(self.data_2_hex_right) == u"11" else (u"c" if str(self.data_2_hex_right) == u"12" else (u"d" if str(self.data_2_hex_right) == u"13" else (u"e" if str(self.data_2_hex_right) == u"14" else (u"f" if str(self.data_2_hex_right) == u"15" else str(self.data_2_hex_right)))))))
            return self._m_data_2_hex_right_digit if hasattr(self, '_m_data_2_hex_right_digit') else None

        @property
        def batt_v_hex_right_digit(self):
            if hasattr(self, '_m_batt_v_hex_right_digit'):
                return self._m_batt_v_hex_right_digit if hasattr(self, '_m_batt_v_hex_right_digit') else None

            self._m_batt_v_hex_right_digit = (u"a" if str(self.batt_v_hex_right) == u"10" else (u"b" if str(self.batt_v_hex_right) == u"11" else (u"c" if str(self.batt_v_hex_right) == u"12" else (u"d" if str(self.batt_v_hex_right) == u"13" else (u"e" if str(self.batt_v_hex_right) == u"14" else (u"f" if str(self.batt_v_hex_right) == u"15" else str(self.batt_v_hex_right)))))))
            return self._m_batt_v_hex_right_digit if hasattr(self, '_m_batt_v_hex_right_digit') else None

        @property
        def batt_t_hex_left(self):
            if hasattr(self, '_m_batt_t_hex_left'):
                return self._m_batt_t_hex_left if hasattr(self, '_m_batt_t_hex_left') else None

            self._m_batt_t_hex_left = self.cw_batt_t // 16
            return self._m_batt_t_hex_left if hasattr(self, '_m_batt_t_hex_left') else None

        @property
        def raw_i_hex_left(self):
            if hasattr(self, '_m_raw_i_hex_left'):
                return self._m_raw_i_hex_left if hasattr(self, '_m_raw_i_hex_left') else None

            self._m_raw_i_hex_left = self.cw_raw_i // 16
            return self._m_raw_i_hex_left if hasattr(self, '_m_raw_i_hex_left') else None

        @property
        def bpb_t_hex_right(self):
            if hasattr(self, '_m_bpb_t_hex_right'):
                return self._m_bpb_t_hex_right if hasattr(self, '_m_bpb_t_hex_right') else None

            self._m_bpb_t_hex_right = (self.cw_bpb_t % 16)
            return self._m_bpb_t_hex_right if hasattr(self, '_m_bpb_t_hex_right') else None

        @property
        def data_3_hex_left(self):
            if hasattr(self, '_m_data_3_hex_left'):
                return self._m_data_3_hex_left if hasattr(self, '_m_data_3_hex_left') else None

            self._m_data_3_hex_left = self.data_3_dec // 16
            return self._m_data_3_hex_left if hasattr(self, '_m_data_3_hex_left') else None

        @property
        def bpb_t_hex_left_digit(self):
            if hasattr(self, '_m_bpb_t_hex_left_digit'):
                return self._m_bpb_t_hex_left_digit if hasattr(self, '_m_bpb_t_hex_left_digit') else None

            self._m_bpb_t_hex_left_digit = (u"a" if str(self.bpb_t_hex_left) == u"10" else (u"b" if str(self.bpb_t_hex_left) == u"11" else (u"c" if str(self.bpb_t_hex_left) == u"12" else (u"d" if str(self.bpb_t_hex_left) == u"13" else (u"e" if str(self.bpb_t_hex_left) == u"14" else (u"f" if str(self.bpb_t_hex_left) == u"15" else str(self.bpb_t_hex_left)))))))
            return self._m_bpb_t_hex_left_digit if hasattr(self, '_m_bpb_t_hex_left_digit') else None

        @property
        def bpb_t_hex_left(self):
            if hasattr(self, '_m_bpb_t_hex_left'):
                return self._m_bpb_t_hex_left if hasattr(self, '_m_bpb_t_hex_left') else None

            self._m_bpb_t_hex_left = self.cw_bpb_t // 16
            return self._m_bpb_t_hex_left if hasattr(self, '_m_bpb_t_hex_left') else None

        @property
        def cw_beacon(self):
            if hasattr(self, '_m_cw_beacon'):
                return self._m_cw_beacon if hasattr(self, '_m_cw_beacon') else None

            self._m_cw_beacon = self.batt_v_hex + self.batt_i_hex + self.batt_t_hex + self.bpb_t_hex + self.raw_i_hex + self.data_1_hex + self.data_2_hex + self.data_3_hex
            return self._m_cw_beacon if hasattr(self, '_m_cw_beacon') else None

        @property
        def data_3_hex_left_digit(self):
            if hasattr(self, '_m_data_3_hex_left_digit'):
                return self._m_data_3_hex_left_digit if hasattr(self, '_m_data_3_hex_left_digit') else None

            self._m_data_3_hex_left_digit = (u"a" if str(self.data_3_hex_left) == u"10" else (u"b" if str(self.data_3_hex_left) == u"11" else (u"c" if str(self.data_3_hex_left) == u"12" else (u"d" if str(self.data_3_hex_left) == u"13" else (u"e" if str(self.data_3_hex_left) == u"14" else (u"f" if str(self.data_3_hex_left) == u"15" else str(self.data_3_hex_left)))))))
            return self._m_data_3_hex_left_digit if hasattr(self, '_m_data_3_hex_left_digit') else None

        @property
        def data_3_dec(self):
            if hasattr(self, '_m_data_3_dec'):
                return self._m_data_3_dec if hasattr(self, '_m_data_3_dec') else None

            self._m_data_3_dec = (((((self.data_3_dec_value_1 + self.data_3_dec_value_2) + self.data_3_dec_value_3) + self.data_3_dec_value_4) + self.data_3_dec_value_5) + self.data_3_dec_value_6)
            return self._m_data_3_dec if hasattr(self, '_m_data_3_dec') else None

        @property
        def batt_t_hex_right_digit(self):
            if hasattr(self, '_m_batt_t_hex_right_digit'):
                return self._m_batt_t_hex_right_digit if hasattr(self, '_m_batt_t_hex_right_digit') else None

            self._m_batt_t_hex_right_digit = (u"a" if str(self.batt_t_hex_right) == u"10" else (u"b" if str(self.batt_t_hex_right) == u"11" else (u"c" if str(self.batt_t_hex_right) == u"12" else (u"d" if str(self.batt_t_hex_right) == u"13" else (u"e" if str(self.batt_t_hex_right) == u"14" else (u"f" if str(self.batt_t_hex_right) == u"15" else str(self.batt_t_hex_right)))))))
            return self._m_batt_t_hex_right_digit if hasattr(self, '_m_batt_t_hex_right_digit') else None

        @property
        def data_1_dec_value_3(self):
            if hasattr(self, '_m_data_1_dec_value_3'):
                return self._m_data_1_dec_value_3 if hasattr(self, '_m_data_1_dec_value_3') else None

            self._m_data_1_dec_value_3 = (32 if int(self.cw_power_com) == 1 else 0)
            return self._m_data_1_dec_value_3 if hasattr(self, '_m_data_1_dec_value_3') else None

        @property
        def data_2_hex(self):
            if hasattr(self, '_m_data_2_hex'):
                return self._m_data_2_hex if hasattr(self, '_m_data_2_hex') else None

            self._m_data_2_hex = self.data_2_hex_left_digit + self.data_2_hex_right_digit
            return self._m_data_2_hex if hasattr(self, '_m_data_2_hex') else None

        @property
        def batt_i_hex_right(self):
            if hasattr(self, '_m_batt_i_hex_right'):
                return self._m_batt_i_hex_right if hasattr(self, '_m_batt_i_hex_right') else None

            self._m_batt_i_hex_right = (self.cw_batt_i % 16)
            return self._m_batt_i_hex_right if hasattr(self, '_m_batt_i_hex_right') else None

        @property
        def raw_i_hex_right(self):
            if hasattr(self, '_m_raw_i_hex_right'):
                return self._m_raw_i_hex_right if hasattr(self, '_m_raw_i_hex_right') else None

            self._m_raw_i_hex_right = (self.cw_raw_i % 16)
            return self._m_raw_i_hex_right if hasattr(self, '_m_raw_i_hex_right') else None

        @property
        def data_1_dec_value_8(self):
            if hasattr(self, '_m_data_1_dec_value_8'):
                return self._m_data_1_dec_value_8 if hasattr(self, '_m_data_1_dec_value_8') else None

            self._m_data_1_dec_value_8 = (1 if int(self.cw_sap_minus_z) == 1 else 0)
            return self._m_data_1_dec_value_8 if hasattr(self, '_m_data_1_dec_value_8') else None

        @property
        def raw_i_hex_right_digit(self):
            if hasattr(self, '_m_raw_i_hex_right_digit'):
                return self._m_raw_i_hex_right_digit if hasattr(self, '_m_raw_i_hex_right_digit') else None

            self._m_raw_i_hex_right_digit = (u"a" if str(self.raw_i_hex_right) == u"10" else (u"b" if str(self.raw_i_hex_right) == u"11" else (u"c" if str(self.raw_i_hex_right) == u"12" else (u"d" if str(self.raw_i_hex_right) == u"13" else (u"e" if str(self.raw_i_hex_right) == u"14" else (u"f" if str(self.raw_i_hex_right) == u"15" else str(self.raw_i_hex_right)))))))
            return self._m_raw_i_hex_right_digit if hasattr(self, '_m_raw_i_hex_right_digit') else None

        @property
        def data_1_hex(self):
            if hasattr(self, '_m_data_1_hex'):
                return self._m_data_1_hex if hasattr(self, '_m_data_1_hex') else None

            self._m_data_1_hex = self.data_1_hex_left_digit + self.data_1_hex_right_digit
            return self._m_data_1_hex if hasattr(self, '_m_data_1_hex') else None

        @property
        def batt_v_hex_left(self):
            if hasattr(self, '_m_batt_v_hex_left'):
                return self._m_batt_v_hex_left if hasattr(self, '_m_batt_v_hex_left') else None

            self._m_batt_v_hex_left = self.cw_batt_v // 16
            return self._m_batt_v_hex_left if hasattr(self, '_m_batt_v_hex_left') else None

        @property
        def bpb_t_hex(self):
            if hasattr(self, '_m_bpb_t_hex'):
                return self._m_bpb_t_hex if hasattr(self, '_m_bpb_t_hex') else None

            self._m_bpb_t_hex = self.bpb_t_hex_left_digit + self.bpb_t_hex_right_digit
            return self._m_bpb_t_hex if hasattr(self, '_m_bpb_t_hex') else None

        @property
        def batt_v_hex_right(self):
            if hasattr(self, '_m_batt_v_hex_right'):
                return self._m_batt_v_hex_right if hasattr(self, '_m_batt_v_hex_right') else None

            self._m_batt_v_hex_right = (self.cw_batt_v % 16)
            return self._m_batt_v_hex_right if hasattr(self, '_m_batt_v_hex_right') else None

        @property
        def data_2_dec_value_3(self):
            if hasattr(self, '_m_data_2_dec_value_3'):
                return self._m_data_2_dec_value_3 if hasattr(self, '_m_data_2_dec_value_3') else None

            self._m_data_2_dec_value_3 = (1 if int(self.cw_kill_sw) == 1 else 0)
            return self._m_data_2_dec_value_3 if hasattr(self, '_m_data_2_dec_value_3') else None

        @property
        def batt_v_hex(self):
            if hasattr(self, '_m_batt_v_hex'):
                return self._m_batt_v_hex if hasattr(self, '_m_batt_v_hex') else None

            self._m_batt_v_hex = self.batt_v_hex_left_digit + self.batt_v_hex_right_digit
            return self._m_batt_v_hex if hasattr(self, '_m_batt_v_hex') else None

        @property
        def batt_i_hex_left(self):
            if hasattr(self, '_m_batt_i_hex_left'):
                return self._m_batt_i_hex_left if hasattr(self, '_m_batt_i_hex_left') else None

            self._m_batt_i_hex_left = self.cw_batt_i // 16
            return self._m_batt_i_hex_left if hasattr(self, '_m_batt_i_hex_left') else None

        @property
        def data_1_hex_right_digit(self):
            if hasattr(self, '_m_data_1_hex_right_digit'):
                return self._m_data_1_hex_right_digit if hasattr(self, '_m_data_1_hex_right_digit') else None

            self._m_data_1_hex_right_digit = (u"a" if str(self.data_1_hex_right) == u"10" else (u"b" if str(self.data_1_hex_right) == u"11" else (u"c" if str(self.data_1_hex_right) == u"12" else (u"d" if str(self.data_1_hex_right) == u"13" else (u"e" if str(self.data_1_hex_right) == u"14" else (u"f" if str(self.data_1_hex_right) == u"15" else str(self.data_1_hex_right)))))))
            return self._m_data_1_hex_right_digit if hasattr(self, '_m_data_1_hex_right_digit') else None

        @property
        def data_1_hex_right(self):
            if hasattr(self, '_m_data_1_hex_right'):
                return self._m_data_1_hex_right if hasattr(self, '_m_data_1_hex_right') else None

            self._m_data_1_hex_right = (self.data_1_dec % 16)
            return self._m_data_1_hex_right if hasattr(self, '_m_data_1_hex_right') else None

        @property
        def data_3_dec_value_5(self):
            if hasattr(self, '_m_data_3_dec_value_5'):
                return self._m_data_3_dec_value_5 if hasattr(self, '_m_data_3_dec_value_5') else None

            self._m_data_3_dec_value_5 = (4 if int(self.cw_aprs_flag) == 1 else 0)
            return self._m_data_3_dec_value_5 if hasattr(self, '_m_data_3_dec_value_5') else None


    class Hk(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.hk_callsign_header = (self._io.read_bytes(14)).decode(u"ASCII")
            if not self.hk_callsign_header == u"JS1YMY0JS1YNI0":
                raise kaitaistruct.ValidationNotEqualError(u"JS1YMY0JS1YNI0", self.hk_callsign_header, self._io, u"/types/hk/seq/0")
            self.hk_header_last_part_0 = self._io.read_u2be()
            if not self.hk_header_last_part_0 == 16112:
                raise kaitaistruct.ValidationNotEqualError(16112, self.hk_header_last_part_0, self._io, u"/types/hk/seq/1")
            self.hk_header_last_part_1 = self._io.read_u2be()
            if not self.hk_header_last_part_1 == 21265:
                raise kaitaistruct.ValidationNotEqualError(21265, self.hk_header_last_part_1, self._io, u"/types/hk/seq/2")
            self.hk_header_last_part_2 = self._io.read_u2be()
            if not self.hk_header_last_part_2 == 65280:
                raise kaitaistruct.ValidationNotEqualError(65280, self.hk_header_last_part_2, self._io, u"/types/hk/seq/3")
            self.hk_header_last_part_3 = self._io.read_u2be()
            if not self.hk_header_last_part_3 == 1:
                raise kaitaistruct.ValidationNotEqualError(1, self.hk_header_last_part_3, self._io, u"/types/hk/seq/4")
            self.hk_header = self._io.read_u1()
            if not self.hk_header == 51:
                raise kaitaistruct.ValidationNotEqualError(51, self.hk_header, self._io, u"/types/hk/seq/5")
            self.seconds = self._io.read_u1()
            self.minutes = self._io.read_u1()
            self.hours = self._io.read_u1()
            self.days = self._io.read_u2be()
            self.n_a = self._io.read_u2be()
            self.temp_minus_x = self._io.read_u2be()
            self.temp_plus_y = self._io.read_u2be()
            self.temp_minus_y = self._io.read_u2be()
            self.temp_plus_z = self._io.read_u2be()
            self.temp_minus_z = self._io.read_u2be()
            self.bpb_t = self._io.read_u2be()
            self.voltage_minus_x = self._io.read_u2be()
            self.voltage_plus_y = self._io.read_u2be()
            self.voltage_minus_y = self._io.read_u2be()
            self.voltage_plus_z = self._io.read_u2be()
            self.voltage_minus_z = self._io.read_u2be()
            self.current_minus_x = self._io.read_u1()
            self.current_plus_y = self._io.read_u1()
            self.current_minus_y = self._io.read_u1()
            self.current_plus_z = self._io.read_u1()
            self.current_minus_z = self._io.read_u1()
            self.batt_t = self._io.read_u1()
            self.batt_v = self._io.read_u1()
            self.batt_i = self._io.read_u2be()
            self.raw_v = self._io.read_u1()
            self.raw_i = self._io.read_u2be()
            self.src_v = self._io.read_u1()
            self.src_i = self._io.read_u2be()
            self.kill_sw = self._io.read_u1()
            self.raw_v_1 = self._io.read_u1()
            self.q3v3_1_i = self._io.read_u1()
            self.q3v3_2_i = self._io.read_u1()
            self.com_i = self._io.read_u1()
            self.ant_dep_i = self._io.read_u1()
            self.q5v0_i = self._io.read_u1()
            self.reset_raw_v_mon = self._io.read_bits_int_be(1) != 0
            self.power_com = self._io.read_bits_int_be(1) != 0
            self.power_5v0 = self._io.read_bits_int_be(1) != 0
            self.dcdc_3v3_1 = self._io.read_bits_int_be(1) != 0
            self.pwr_ant_dep = self._io.read_bits_int_be(1) != 0
            self.pwr_3v3_2 = self._io.read_bits_int_be(1) != 0
            self.pwr_3v3_1 = self._io.read_bits_int_be(1) != 0
            self.dcdc_5v0 = self._io.read_bits_int_be(1) != 0
            self.dcdc_3v3_2 = self._io.read_bits_int_be(1) != 0
            self.pwr_compic = self._io.read_bits_int_be(1) != 0
            self.pwr_mainpic = self._io.read_bits_int_be(1) != 0
            self.empty = self._io.read_bits_int_be(5)
            self._io.align_to_byte()
            self.mp_reset_counter = self._io.read_u1()
            self.rssi = self._io.read_u1()
            self.com_t = self._io.read_u1()
            self.com_seq_counter = self._io.read_u1()
            self.cmd_uplink_counter = self._io.read_u1()
            self.mis_ack = self._io.read_u1()
            self.n_a_2 = self._io.read_u1()
            self.current_mis = self._io.read_u1()
            self.mis_counter = self._io.read_u1()
            self.checksum = self._io.read_u1()

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"HK" if 0 == 0 else u"HK")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None


    class Digi(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = Sakura.Digi.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Sakura.Digi.Ax25Header(self._io, self, self._root)
                _on = (self.ax25_header.ctl & 19)
                if _on == 0:
                    self.payload = Sakura.Digi.IFrame(self._io, self, self._root)
                elif _on == 3:
                    self.payload = Sakura.Digi.UiFrame(self._io, self, self._root)
                elif _on == 19:
                    self.payload = Sakura.Digi.UiFrame(self._io, self, self._root)
                elif _on == 16:
                    self.payload = Sakura.Digi.IFrame(self._io, self, self._root)
                elif _on == 18:
                    self.payload = Sakura.Digi.IFrame(self._io, self, self._root)
                elif _on == 2:
                    self.payload = Sakura.Digi.IFrame(self._io, self, self._root)


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = Sakura.Digi.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = Sakura.Digi.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = Sakura.Digi.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = Sakura.Digi.SsidMask(self._io, self, self._root)
                if (self.src_ssid_raw.ssid_mask & 1) == 0:
                    self.repeater = Sakura.Digi.Repeater(self._io, self, self._root)

                self.ctl = self._io.read_u1()


        class UiFrame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pid = self._io.read_u1()
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Sakura.Digi.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class IFrame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pid = self._io.read_u1()
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Sakura.Digi.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid if hasattr(self, '_m_ssid') else None

                self._m_ssid = ((self.ssid_mask & 15) >> 1)
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            @property
            def hbit(self):
                if hasattr(self, '_m_hbit'):
                    return self._m_hbit if hasattr(self, '_m_hbit') else None

                self._m_hbit = ((self.ssid_mask & 128) >> 7)
                return self._m_hbit if hasattr(self, '_m_hbit') else None


        class Repeaters(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_callsign_raw = Sakura.Digi.CallsignRaw(self._io, self, self._root)
                self.rpt_ssid_raw = Sakura.Digi.SsidMask(self._io, self, self._root)


        class Repeater(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_instance = []
                i = 0
                while True:
                    _ = Sakura.Digi.Repeaters(self._io, self, self._root)
                    self.rpt_instance.append(_)
                    if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                        break
                    i += 1


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Sakura.Digi.Callsign(_io__raw_callsign_ror, self, self._root)


        class Ax25InfoData(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.data_monitor = (self._io.read_bytes_full()).decode(u"utf-8")


        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None

            self._m_beacon_type = (u"DIGI" if 0 == 0 else u"DIGI")
            return self._m_beacon_type if hasattr(self, '_m_beacon_type') else None


    class SakuraT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 8314044539276959850:
                self.type_check = Sakura.Cw(self._io, self, self._root)
            elif _on == 5355678641493192778:
                self.type_check = Sakura.DigitalBeacon(self._io, self, self._root)
            else:
                self.type_check = Sakura.Digi(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check if hasattr(self, '_m_check') else None

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_check = self._io.read_u8be()
            self._io.seek(_pos)
            return self._m_check if hasattr(self, '_m_check') else None


    class DigitalBeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check_1
            if _on == 204:
                self.type_check_1 = Sakura.Cam(self._io, self, self._root)
            elif _on == 17:
                self.type_check_1 = Sakura.AutoGmskOrHk(self._io, self, self._root)
            else:
                self.type_check_1 = Sakura.Discard(self._io, self, self._root)

        @property
        def check_1(self):
            if hasattr(self, '_m_check_1'):
                return self._m_check_1 if hasattr(self, '_m_check_1') else None

            _pos = self._io.pos()
            self._io.seek(17)
            self._m_check_1 = self._io.read_u1()
            self._io.seek(_pos)
            return self._m_check_1 if hasattr(self, '_m_check_1') else None






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Kashiwa(KaitaiStruct):
    """:field batt_v: kashiwa.type_check.batt_v
    :field batt_i: kashiwa.type_check.batt_i
    :field batt_t: kashiwa.type_check.batt_t
    :field bpb_t: kashiwa.type_check.bpb_t
    :field raw_i: kashiwa.type_check.raw_i
    :field lcl_5v0: kashiwa.type_check.lcl_5v0
    :field lcl_depand: kashiwa.type_check.lcl_depand
    :field lcl_compic: kashiwa.type_check.lcl_compic
    :field sap_minus_x: kashiwa.type_check.sap_minus_x
    :field sap_plus_y: kashiwa.type_check.sap_plus_y
    :field sap_minus_y: kashiwa.type_check.sap_minus_y
    :field sap_plus_z: kashiwa.type_check.sap_plus_z
    :field sap_minus_z: kashiwa.type_check.sap_minus_z
    :field sap_minus_z: kashiwa.type_check.sap_minus_z
    :field reserve: kashiwa.type_check.reserve
    :field reserve_cmd_counter: kashiwa.type_check.reserve_cmd_counter
    :field gmsk_cmd_counter: kashiwa.type_check.gmsk_cmd_counter
    :field kill_counter: kashiwa.type_check.kill_counter
    :field kill_sw: kashiwa.type_check.kill_sw
    :field boss_on_off: kashiwa.type_check.boss_on_off
    :field ack_mis_end: kashiwa.type_check.ack_mis_end
    :field ack_mis_error: kashiwa.type_check.ack_mis_error
    :field ack_mis_mog: kashiwa.type_check.ack_mis_mog
    :field mis_iss: kashiwa.type_check.mis_iss
    :field beacon_field: kashiwa.type_check.beacon_field
    :field dest_callsign: kashiwa.type_check.ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: kashiwa.type_check.ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: kashiwa.type_check.ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: kashiwa.type_check.ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_instance___callsign: kashiwa.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_callsign_raw.callsign_ror.callsign
    :field rpt_instance___ssid: kashiwa.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.ssid
    :field rpt_instance___hbit: kashiwa.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.hbit
    :field ctl: kashiwa.type_check.ax25_frame.ax25_header.ctl
    :field pid: kashiwa.type_check.ax25_frame.payload.pid
    :field monitor: kashiwa.type_check.ax25_frame.payload.ax25_info.data_monitor
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.kashiwa = Kashiwa.KashiwaT(self._io, self, self._root)

    class KashiwaT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 5355678641493134145:
                self.type_check = Kashiwa.Cw(self._io, self, self._root)
            elif _on == 7670528987939498849:
                self.type_check = Kashiwa.Cw(self._io, self, self._root)
            elif _on == 5352306439146123338:
                self.type_check = Kashiwa.SpecialBeacon(self._io, self, self._root)
            else:
                self.type_check = Kashiwa.Aprs(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check if hasattr(self, '_m_check') else None

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_check = self._io.read_u8be()
            self._io.seek(_pos)
            return self._m_check if hasattr(self, '_m_check') else None


    class Cw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign_and_satellite_name = (self._io.read_bytes(13)).decode(u"ASCII")
            if not  ((self.callsign_and_satellite_name == u"JS1YMXKASHIWA") or (self.callsign_and_satellite_name == u"js1ymxkashiwa")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign_and_satellite_name, self._io, u"/types/cw/seq/0")
            self.batt_v = self._io.read_u1()
            self.batt_i = self._io.read_u1()
            self.batt_t = self._io.read_u1()
            self.bpb_t = self._io.read_u1()
            self.raw_i = self._io.read_u1()
            self.lcl_5v0 = self._io.read_bits_int_be(1) != 0
            self.lcl_depand = self._io.read_bits_int_be(1) != 0
            self.lcl_compic = self._io.read_bits_int_be(1) != 0
            self.sap_minus_x = self._io.read_bits_int_be(1) != 0
            self.sap_plus_y = self._io.read_bits_int_be(1) != 0
            self.sap_minus_y = self._io.read_bits_int_be(1) != 0
            self.sap_plus_z = self._io.read_bits_int_be(1) != 0
            self.sap_minus_z = self._io.read_bits_int_be(1) != 0
            self.reserve = self._io.read_bits_int_be(1) != 0
            self.reserve_cmd_counter = self._io.read_bits_int_be(4)
            self.gmsk_cmd_counter = self._io.read_bits_int_be(3)
            self.kill_counter = self._io.read_bits_int_be(2)
            self.kill_sw = self._io.read_bits_int_be(1) != 0
            self.boss_on_off = self._io.read_bits_int_be(1) != 0
            self.ack_mis_end = self._io.read_bits_int_be(1) != 0
            self.ack_mis_error = self._io.read_bits_int_be(1) != 0
            self.ack_mis_mog = self._io.read_bits_int_be(1) != 0
            self.mis_iss = self._io.read_bits_int_be(1) != 0


    class SpecialBeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_source_callsign = (self._io.read_bytes(14)).decode(u"ASCII")
            if not  ((self.dest_source_callsign == u"JG6YBW0JG6YMX0")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.dest_source_callsign, self._io, u"/types/special_beacon/seq/0")
            self.ctl_pid = self._io.read_u2be()
            self.beacon_field = (self._io.read_bytes_full()).decode(u"UTF-16BE")


    class Aprs(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = Kashiwa.Aprs.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Kashiwa.Aprs.Ax25Header(self._io, self, self._root)
                _on = (self.ax25_header.ctl & 19)
                if _on == 0:
                    self.payload = Kashiwa.Aprs.IFrame(self._io, self, self._root)
                elif _on == 3:
                    self.payload = Kashiwa.Aprs.UiFrame(self._io, self, self._root)
                elif _on == 19:
                    self.payload = Kashiwa.Aprs.UiFrame(self._io, self, self._root)
                elif _on == 16:
                    self.payload = Kashiwa.Aprs.IFrame(self._io, self, self._root)
                elif _on == 18:
                    self.payload = Kashiwa.Aprs.IFrame(self._io, self, self._root)
                elif _on == 2:
                    self.payload = Kashiwa.Aprs.IFrame(self._io, self, self._root)


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = Kashiwa.Aprs.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = Kashiwa.Aprs.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = Kashiwa.Aprs.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = Kashiwa.Aprs.SsidMask(self._io, self, self._root)
                if (self.src_ssid_raw.ssid_mask & 1) == 0:
                    self.repeater = Kashiwa.Aprs.Repeater(self._io, self, self._root)

                self.ctl = self._io.read_u1()


        class UiFrame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pid = self._io.read_u1()
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Kashiwa.Aprs.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class IFrame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pid = self._io.read_u1()
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Kashiwa.Aprs.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid if hasattr(self, '_m_ssid') else None

                self._m_ssid = ((self.ssid_mask & 15) >> 1)
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            @property
            def hbit(self):
                if hasattr(self, '_m_hbit'):
                    return self._m_hbit if hasattr(self, '_m_hbit') else None

                self._m_hbit = ((self.ssid_mask & 128) >> 7)
                return self._m_hbit if hasattr(self, '_m_hbit') else None


        class Repeaters(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_callsign_raw = Kashiwa.Aprs.CallsignRaw(self._io, self, self._root)
                self.rpt_ssid_raw = Kashiwa.Aprs.SsidMask(self._io, self, self._root)


        class Repeater(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_instance = []
                i = 0
                while True:
                    _ = Kashiwa.Aprs.Repeaters(self._io, self, self._root)
                    self.rpt_instance.append(_)
                    if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                        break
                    i += 1


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Kashiwa.Aprs.Callsign(_io__raw_callsign_ror, self, self._root)


        class Ax25InfoData(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.data_monitor = (self._io.read_bytes_full()).decode(u"utf-8")




